#!BPY

"""
Name: 'MD2 (.md2) xx'
Blender: 241
Group: 'Export'
Tooltip: 'Export to Quake file format (.md2).'
"""

__author__ = 'Bob Holcomb'
__version__ = '0.17.5'
__url__ = ["Bob's site, http://bane.servebeer.com",
     "Support forum, http://scourage.servebeer.com/phpbb/", "blender", "elysiun"]
__email__ = ["Bob Holcomb, bob_holcomb:hotmail*com", "scripts"]
__bpydoc__ = """\
This script Exports a Quake 2 file (MD2).

 Additional help from: Shadwolf, Skandal, Rojo, Cambo<br>
 Thanks Guys!
"""

# ***** BEGIN GPL LICENSE BLOCK *****
#
# Script copyright (C): Bob Holcomb
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------


import Blender
from Blender import *
from Blender.Draw import *
from Blender.BGL import *
from Blender.Window import *

import struct, string
from types import *



######################################################
# GUI Loader
######################################################

# Export globals
g_filename=Create("tris.md2")
g_frame_filename=Create("default")

g_filename_search=Create("")
g_frame_search=Create("default")

g_texture_path=Create("")

user_frame_list=[]

#Globals
g_scale=Create(1.0)

# Events
EVENT_NOEVENT=1
EVENT_SAVE_MD2=2
EVENT_CHOOSE_FILENAME=3
EVENT_CHOOSE_FRAME=4
EVENT_EXIT=100

######################################################
# Callbacks for Window functions
######################################################
def filename_callback(input_filename):
	global g_filename
	g_filename.val=input_filename

def frame_callback(input_frame):
	global g_frame_filename
	g_frame_filename.val=input_frame

def draw_gui():
	global g_scale
	global g_filename
	global g_frame_filename
	global EVENT_NOEVENT,EVENT_SAVE_MD2,EVENT_CHOOSE_FILENAME,EVENT_CHOOSE_FRAME,EVENT_EXIT
	global g_texture_path

	########## Titles
	glClear(GL_COLOR_BUFFER_BIT)
	glRasterPos2d(10, 120)
	Text("MD2 Export")

	######### Parameters GUI Buttons
	######### MD2 Filename text entry
	g_filename = String("MD2 file to save: ", EVENT_NOEVENT, 10, 75, 210, 18,
                            g_filename.val, 255, "MD2 file to save")
	########## MD2 File Search Button
	Button("Search",EVENT_CHOOSE_FILENAME,220,75,80,18)

	##########  MD2 Frame List Text entry
	g_frame_filename = String("Frame List file to load: ", EVENT_NOEVENT, 10, 55, 210, 18,
                                g_frame_filename.val, 255, "Frame List to load-overrides MD2 defaults")
	########## Frame List Search Button
	Button("Search",EVENT_CHOOSE_FRAME,220,55,80,18)

	##########  Texture path to append
	g_texture_path=String("Texture Path: ", EVENT_NOEVENT, 10,35,210,18,
														g_texture_path.val,255, "Texture path to prepend")


	########## Scale slider-default is 1/8 which is a good scale for md2->blender
	g_scale= Slider("Scale Factor: ", EVENT_NOEVENT, 10, 95, 210, 18,
                    1.0, 0.001, 10.0, 1, "Scale factor for obj Model");

	######### Draw and Exit Buttons
	Button("Export",EVENT_SAVE_MD2 , 10, 10, 80, 18)
	Button("Exit",EVENT_EXIT , 170, 10, 80, 18)

def event(evt, val):
	if (evt == QKEY and not val):
		Exit()

def bevent(evt):
	global g_filename
	global g_frame_filename
	global EVENT_NOEVENT,EVENT_SAVE_MD2,EVENT_EXIT

	######### Manages GUI events
	if (evt==EVENT_EXIT):
		Blender.Draw.Exit()
	elif (evt==EVENT_CHOOSE_FILENAME):
		FileSelector(filename_callback, "MD2 File Selection")
	elif (evt==EVENT_CHOOSE_FRAME):
		FileSelector(frame_callback, "Frame Selection")
	elif (evt==EVENT_SAVE_MD2):
		if (g_filename.val == "model"):
			save_md2("blender.md2")
			Blender.Draw.Exit()
			return
		else:
			save_md2(g_filename.val)
			Blender.Draw.Exit()
			return

Register(draw_gui, event, bevent)

######################################################
# MD2 Model Constants
######################################################
MD2_MAX_TRIANGLES=4096
MD2_MAX_VERTICES=2048
MD2_MAX_TEXCOORDS=2048
MD2_MAX_FRAMES=512
MD2_MAX_SKINS=32
MD2_MAX_FRAMESIZE=(MD2_MAX_VERTICES * 4 + 128)

MD2_FRAME_NAME_LIST=(("stand",1,40),
					("run",41,46),
					("attack",47,54),
					("pain1",55,58),
					("pain2",59,62),
					("pain3",63,66),
					("jump",67,72),
					("flip",73,84),
					("salute", 85,95),
					("taunt",96,112),
					("wave",113,123),
					("point",124,135),
					("crstnd",136,154),
					("crwalk",155,160),
					("crattack",161,169),
					("crpain",170,173),
					("crdeath",174,178),
					("death1",179,184),
					("death2",185,190),
					("death3",191,198))
					#198 frames

MD2_NORMALS=((-0.525731, 0.000000, 0.850651),
			(-0.442863, 0.238856, 0.864188),
			(-0.295242, 0.000000, 0.955423),
			(-0.309017, 0.500000, 0.809017),
			(-0.162460, 0.262866, 0.951056),
			(0.000000, 0.000000, 1.000000),
			(0.000000, 0.850651, 0.525731),
			(-0.147621, 0.716567, 0.681718),
			(0.147621, 0.716567, 0.681718),
			(0.000000, 0.525731, 0.850651),
			(0.309017, 0.500000, 0.809017),
			(0.525731, 0.000000, 0.850651),
			(0.295242, 0.000000, 0.955423),
			(0.442863, 0.238856, 0.864188),
			(0.162460, 0.262866, 0.951056),
			(-0.681718, 0.147621, 0.716567),
			(-0.809017, 0.309017, 0.500000),
			(-0.587785, 0.425325, 0.688191),
			(-0.850651, 0.525731, 0.000000),
			(-0.864188, 0.442863, 0.238856),
			(-0.716567, 0.681718, 0.147621),
			(-0.688191, 0.587785, 0.425325),
			(-0.500000, 0.809017, 0.309017),
			(-0.238856, 0.864188, 0.442863),
			(-0.425325, 0.688191, 0.587785),
			(-0.716567, 0.681718, -0.147621),
			(-0.500000, 0.809017, -0.309017),
			(-0.525731, 0.850651, 0.000000),
			(0.000000, 0.850651, -0.525731),
			(-0.238856, 0.864188, -0.442863),
			(0.000000, 0.955423, -0.295242),
			(-0.262866, 0.951056, -0.162460),
			(0.000000, 1.000000, 0.000000),
			(0.000000, 0.955423, 0.295242),
			(-0.262866, 0.951056, 0.162460),
			(0.238856, 0.864188, 0.442863),
			(0.262866, 0.951056, 0.162460),
			(0.500000, 0.809017, 0.309017),
			(0.238856, 0.864188, -0.442863),
			(0.262866, 0.951056, -0.162460),
			(0.500000, 0.809017, -0.309017),
			(0.850651, 0.525731, 0.000000),
			(0.716567, 0.681718, 0.147621),
			(0.716567, 0.681718, -0.147621),
			(0.525731, 0.850651, 0.000000),
			(0.425325, 0.688191, 0.587785),
			(0.864188, 0.442863, 0.238856),
			(0.688191, 0.587785, 0.425325),
			(0.809017, 0.309017, 0.500000),
			(0.681718, 0.147621, 0.716567),
			(0.587785, 0.425325, 0.688191),
			(0.955423, 0.295242, 0.000000),
			(1.000000, 0.000000, 0.000000),
			(0.951056, 0.162460, 0.262866),
			(0.850651, -0.525731, 0.000000),
			(0.955423, -0.295242, 0.000000),
			(0.864188, -0.442863, 0.238856),
			(0.951056, -0.162460, 0.262866),
			(0.809017, -0.309017, 0.500000),
			(0.681718, -0.147621, 0.716567),
			(0.850651, 0.000000, 0.525731),
			(0.864188, 0.442863, -0.238856),
			(0.809017, 0.309017, -0.500000),
			(0.951056, 0.162460, -0.262866),
			(0.525731, 0.000000, -0.850651),
			(0.681718, 0.147621, -0.716567),
			(0.681718, -0.147621, -0.716567),
			(0.850651, 0.000000, -0.525731),
			(0.809017, -0.309017, -0.500000),
			(0.864188, -0.442863, -0.238856),
			(0.951056, -0.162460, -0.262866),
			(0.147621, 0.716567, -0.681718),
			(0.309017, 0.500000, -0.809017),
			(0.425325, 0.688191, -0.587785),
			(0.442863, 0.238856, -0.864188),
			(0.587785, 0.425325, -0.688191),
			(0.688191, 0.587785, -0.425325),
			(-0.147621, 0.716567, -0.681718),
			(-0.309017, 0.500000, -0.809017),
			(0.000000, 0.525731, -0.850651),
			(-0.525731, 0.000000, -0.850651),
			(-0.442863, 0.238856, -0.864188),
			(-0.295242, 0.000000, -0.955423),
			(-0.162460, 0.262866, -0.951056),
			(0.000000, 0.000000, -1.000000),
			(0.295242, 0.000000, -0.955423),
			(0.162460, 0.262866, -0.951056),
			(-0.442863, -0.238856, -0.864188),
			(-0.309017, -0.500000, -0.809017),
			(-0.162460, -0.262866, -0.951056),
			(0.000000, -0.850651, -0.525731),
			(-0.147621, -0.716567, -0.681718),
			(0.147621, -0.716567, -0.681718),
			(0.000000, -0.525731, -0.850651),
			(0.309017, -0.500000, -0.809017),
			(0.442863, -0.238856, -0.864188),
			(0.162460, -0.262866, -0.951056),
			(0.238856, -0.864188, -0.442863),
			(0.500000, -0.809017, -0.309017),
			(0.425325, -0.688191, -0.587785),
			(0.716567, -0.681718, -0.147621),
			(0.688191, -0.587785, -0.425325),
			(0.587785, -0.425325, -0.688191),
			(0.000000, -0.955423, -0.295242),
			(0.000000, -1.000000, 0.000000),
			(0.262866, -0.951056, -0.162460),
			(0.000000, -0.850651, 0.525731),
			(0.000000, -0.955423, 0.295242),
			(0.238856, -0.864188, 0.442863),
			(0.262866, -0.951056, 0.162460),
			(0.500000, -0.809017, 0.309017),
			(0.716567, -0.681718, 0.147621),
			(0.525731, -0.850651, 0.000000),
			(-0.238856, -0.864188, -0.442863),
			(-0.500000, -0.809017, -0.309017),
			(-0.262866, -0.951056, -0.162460),
			(-0.850651, -0.525731, 0.000000),
			(-0.716567, -0.681718, -0.147621),
			(-0.716567, -0.681718, 0.147621),
			(-0.525731, -0.850651, 0.000000),
			(-0.500000, -0.809017, 0.309017),
			(-0.238856, -0.864188, 0.442863),
			(-0.262866, -0.951056, 0.162460),
			(-0.864188, -0.442863, 0.238856),
			(-0.809017, -0.309017, 0.500000),
			(-0.688191, -0.587785, 0.425325),
			(-0.681718, -0.147621, 0.716567),
			(-0.442863, -0.238856, 0.864188),
			(-0.587785, -0.425325, 0.688191),
			(-0.309017, -0.500000, 0.809017),
			(-0.147621, -0.716567, 0.681718),
			(-0.425325, -0.688191, 0.587785),
			(-0.162460, -0.262866, 0.951056),
			(0.442863, -0.238856, 0.864188),
			(0.162460, -0.262866, 0.951056),
			(0.309017, -0.500000, 0.809017),
			(0.147621, -0.716567, 0.681718),
			(0.000000, -0.525731, 0.850651),
			(0.425325, -0.688191, 0.587785),
			(0.587785, -0.425325, 0.688191),
			(0.688191, -0.587785, 0.425325),
			(-0.955423, 0.295242, 0.000000),
			(-0.951056, 0.162460, 0.262866),
			(-1.000000, 0.000000, 0.000000),
			(-0.850651, 0.000000, 0.525731),
			(-0.955423, -0.295242, 0.000000),
			(-0.951056, -0.162460, 0.262866),
			(-0.864188, 0.442863, -0.238856),
			(-0.951056, 0.162460, -0.262866),
			(-0.809017, 0.309017, -0.500000),
			(-0.864188, -0.442863, -0.238856),
			(-0.951056, -0.162460, -0.262866),
			(-0.809017, -0.309017, -0.500000),
			(-0.681718, 0.147621, -0.716567),
			(-0.681718, -0.147621, -0.716567),
			(-0.850651, 0.000000, -0.525731),
			(-0.688191, 0.587785, -0.425325),
			(-0.587785, 0.425325, -0.688191),
			(-0.425325, 0.688191, -0.587785),
			(-0.425325, -0.688191, -0.587785),
			(-0.587785, -0.425325, -0.688191),
			(-0.688191, -0.587785, -0.425325))


######################################################
# MD2 data structures
######################################################
class md2_point:
	vertices=[]
	lightnormalindex=0
	binary_format="<3BB"
	def __init__(self):
		self.vertices=[0]*3
		self.lightnormalindex=0
	def save(self, file):
		temp_data=[0]*4
		temp_data[0]=self.vertices[0]
		temp_data[1]=self.vertices[1]
		temp_data[2]=self.vertices[2]
		temp_data[3]=self.lightnormalindex
		data=struct.pack(self.binary_format, temp_data[0], temp_data[1], temp_data[2], temp_data[3])
		file.write(data)
	def dump(self):
		print "MD2 Point Structure"
		print "vertex X: ", self.vertices[0]
		print "vertex Y: ", self.vertices[1]
		print "vertex Z: ", self.vertices[2]
		print "lightnormalindex: ",self.lightnormalindex
		print ""

class md2_face:
	vertex_index=[]
	texture_index=[]
	binary_format="<3h3h"
	def __init__(self):
		self.vertex_index = [ 0, 0, 0 ]
		self.texture_index = [ 0, 0, 0]
	def save(self, file):
		temp_data=[0]*6
		#swap vertices around so they draw right
		temp_data[0]=self.vertex_index[0]
		temp_data[1]=self.vertex_index[2]
		temp_data[2]=self.vertex_index[1]
		#swap texture vertices around so they draw right
		temp_data[3]=self.texture_index[0]
		temp_data[4]=self.texture_index[2]
		temp_data[5]=self.texture_index[1]
		data=struct.pack(self.binary_format,temp_data[0],temp_data[1],temp_data[2],temp_data[3],temp_data[4],temp_data[5])
		file.write(data)
	def dump (self):
		print "MD2 Face Structure"
		print "vertex 1 index: ", self.vertex_index[0]
		print "vertex 2 index: ", self.vertex_index[1]
		print "vertex 3 index: ", self.vertex_index[2]
		print "texture 1 index: ", self.texture_index[0]
		print "texture 2 index: ", self.texture_index[1]
		print "texture 3 index: ", self.texture_index[2]
		print ""

class md2_tex_coord:
	u=0
	v=0
	binary_format="<2h"
	def __init__(self):
		self.u=0
		self.v=0
	def save(self, file):
		temp_data=[0]*2
		temp_data[0]=self.u
		temp_data[1]=self.v
		data=struct.pack(self.binary_format, temp_data[0], temp_data[1])
		file.write(data)
	def dump (self):
		print "MD2 Texture Coordinate Structure"
		print "texture coordinate u: ",self.u
		print "texture coordinate v: ",self.v
		print ""

class md2_GL_command:
	s=0.0
	t=0.0
	vert_index=0
	binary_format="<2fi"

	def __init__(self):
		self.s=0.0
		self.t=0.0
		vert_index=0
	def save(self,file):
		temp_data=[0]*3
		temp_data[0]=float(self.s)
		temp_data[1]=float(self.t)
		temp_data[2]=self.vert_index
		data=struct.pack(self.binary_format, temp_data[0],temp_data[1],temp_data[2])
		file.write(data)
	def dump (self):
		print "MD2 OpenGL Command"
		print "s: ", self.s
		print "t: ", self.t
		print "Vertex Index: ", self.vert_index
		print ""

class md2_GL_cmd_list:
	num=0
	cmd_list=[]
	binary_format="<i"

	def __init__(self):
		self.num=0
		self.cmd_list=[]

	def save(self,file):
		data=struct.pack(self.binary_format, self.num)
		file.write(data)
		for cmd in self.cmd_list:
			cmd.save(file)
	def dump(self):
		print "MD2 OpenGL Command List"
		print "number: ", self.num
		for cmd in self.cmd_list:
			cmd.dump()
		print ""

class md2_skin:
	name=""
	binary_format="<64s"
	def __init__(self):
		self.name=""
	def save(self, file):
		temp_data=self.name
		data=struct.pack(self.binary_format, temp_data)
		file.write(data)
	def dump (self):
		print "MD2 Skin"
		print "skin name: ",self.name
		print ""

class md2_frame:
	scale=[]
	translate=[]
	name=[]
	vertices=[]
	binary_format="<3f3f16s"

	def __init__(self):
		self.scale=[0.0]*3
		self.translate=[0.0]*3
		self.name=""
		self.vertices=[]
	def save(self, file):
		temp_data=[0]*7
		temp_data[0]=float(self.scale[0])
		temp_data[1]=float(self.scale[1])
		temp_data[2]=float(self.scale[2])
		temp_data[3]=float(self.translate[0])
		temp_data[4]=float(self.translate[1])
		temp_data[5]=float(self.translate[2])
		temp_data[6]=self.name
		data=struct.pack(self.binary_format, temp_data[0],temp_data[1],temp_data[2],temp_data[3],temp_data[4],temp_data[5],temp_data[6])
		file.write(data)
	def dump (self):
		print "MD2 Frame"
		print "scale x: ",self.scale[0]
		print "scale y: ",self.scale[1]
		print "scale z: ",self.scale[2]
		print "translate x: ",self.translate[0]
		print "translate y: ",self.translate[1]
		print "translate z: ",self.translate[2]
		print "name: ",self.name
		print ""

class md2_obj:
	#Header Structure
	ident=0				#int 0	This is used to identify the file
	version=0			#int 1	The version number of the file (Must be 8)
	skin_width=0		#int 2	The skin width in pixels
	skin_height=0		#int 3	The skin height in pixels
	frame_size=0		#int 4	The size in bytes the frames are
	num_skins=0			#int 5	The number of skins associated with the model
	num_vertices=0		#int 6	The number of vertices (constant for each frame)
	num_tex_coords=0	#int 7	The number of texture coordinates
	num_faces=0			#int 8	The number of faces (polygons)
	num_GL_commands=0	#int 9	The number of gl commands
	num_frames=0		#int 10	The number of animation frames
	offset_skins=0		#int 11	The offset in the file for the skin data
	offset_tex_coords=0	#int 12	The offset in the file for the texture data
	offset_faces=0		#int 13	The offset in the file for the face data
	offset_frames=0		#int 14	The offset in the file for the frames data
	offset_GL_commands=0#int 15	The offset in the file for the gl commands data
	offset_end=0		#int 16	The end of the file offset
	binary_format="<17i"  #little-endian (<), 17 integers (17i)
	#md2 data objects
	tex_coords=[]
	faces=[]
	frames=[]
	skins=[]
	GL_commands=[]

	def __init__ (self):
		self.tex_coords=[]
		self.faces=[]
		self.frames=[]
		self.skins=[]
	def save(self, file):
		temp_data=[0]*17
		temp_data[0]=self.ident
		temp_data[1]=self.version
		temp_data[2]=self.skin_width
		temp_data[3]=self.skin_height
		temp_data[4]=self.frame_size
		temp_data[5]=self.num_skins
		temp_data[6]=self.num_vertices
		temp_data[7]=self.num_tex_coords
		temp_data[8]=self.num_faces
		temp_data[9]=self.num_GL_commands
		temp_data[10]=self.num_frames
		temp_data[11]=self.offset_skins
		temp_data[12]=self.offset_tex_coords
		temp_data[13]=self.offset_faces
		temp_data[14]=self.offset_frames
		temp_data[15]=self.offset_GL_commands
		temp_data[16]=self.offset_end
		data=struct.pack(self.binary_format, temp_data[0],temp_data[1],temp_data[2],temp_data[3],temp_data[4],temp_data[5],temp_data[6],temp_data[7],temp_data[8],temp_data[9],temp_data[10],temp_data[11],temp_data[12],temp_data[13],temp_data[14],temp_data[15],temp_data[16])
		file.write(data)
		#write the skin data
		for skin in self.skins:
			skin.save(file)
		#save the texture coordinates
		for tex_coord in self.tex_coords:
			tex_coord.save(file)
		#save the face info
		for face in self.faces:
			face.save(file)
		#save the frames
		for frame in self.frames:
			frame.save(file)
			for vert in frame.vertices:
				vert.save(file)
		#save the GL command List
		for cmd in self.GL_commands:
			cmd.save(file)
	def dump (self):
		print "Header Information"
		print "ident: ", self.ident
		print "version: ", self.version
		print "skin width: ", self.skin_width
		print "skin height: ", self.skin_height
		print "frame size: ", self.frame_size
		print "number of skins: ", self.num_skins
		print "number of texture coordinates: ", self.num_tex_coords
		print "number of faces: ", self.num_faces
		print "number of frames: ", self.num_frames
		print "number of vertices: ", self.num_vertices
		print "number of GL commands: ",self.num_GL_commands
		print "offset skins: ", self.offset_skins
		print "offset texture coordinates: ", self.offset_tex_coords
		print "offset faces: ", self.offset_faces
		print "offset frames: ",self.offset_frames
		print "offset GL Commands: ",self.offset_GL_commands
		print "offset end: ",self.offset_end
		print ""

######################################################
# Validation
######################################################
def validation(object):
	global user_frame_list

	#move the object to the origin if it's not already there
	if object.loc!=(0.0, 0.0, 0.0):
		object.setLocation(0.0,0.0,0.0)
		print "Model not centered at origin-Centering"
		result=Blender.Draw.PupMenu("Model not centered at origin-Centering")

	#resize the object in case it is not the right size
	if object.size!=(1.0,1.0,1.0):
		object.setSize(1.0,1.0,1.0)
		print "Object is scaled-You should scale the mesh verts, not the object"
		result=Blender.Draw.PupMenu("Object is scaled-You should scale the mesh verts, not the object")

	if object.rot!=(0.0,0.0,0.0):
		object.rot=(0.0,0.0,0.0)
		print "Object is rotated-You should rotate the mesh verts, not the object"
		result=Blender.Draw.PupMenu("Object is rotated-You should rotate the mesh verts, not the object")

	#get access to the mesh data
	mesh=object.getData(False, True) #get the object (not just name) and the Mesh, not NMesh

	#check it's composed of only tri's
	result=0
	for face in mesh.faces:
		if len(face.verts)!=3:
			#select the face for future triangulation
			face.sel=1
			if result==0:  #first time we have this problem, don't pop-up a window every time it finds a quad
			  print "Model not made entirely of triangles"
			  result=Blender.Draw.PupMenu("Model not made entirely out of Triangles-Convert?%t|YES|NO")

	#triangulate or quit
	if result==1:
		#selecting face mode
		Blender.Mesh.Mode(3)

		editmode = Window.EditMode()    # are we in edit mode?  If so ...
		if editmode: Window.EditMode(0) # leave edit mode before getting the mesh

		mesh.quadToTriangle(0) #use closest verticies in breaking a quad
	elif result==2:
		return False #user will fix (I guess)

	#check it has UV coordinates
	if mesh.vertexUV==True:
		print "Vertex UV not supported"
		result=Blender.Draw.PupMenu("Vertex UV not suppored-Use Sticky UV%t|Quit")
		return False

	elif mesh.faceUV==True:
		for face in mesh.faces:
			if(len(face.uv)==3):
				pass
			else:
				print "Model's vertices do not all have UV"
				result=Blender.Draw.PupMenu("Model's vertices do not all have UV%t|Quit")
				return False

	else:
		print "Model does not have UV (face or vertex)"
		result=Blender.Draw.PupMenu("Model does not have UV (face or vertex)%t|Quit")
		return False

	#check it has an associated texture map
	last_face=""
	last_face=mesh.faces[0].image
	if last_face=="":
		print "Model does not have a texture Map"
		result=Blender.Draw.PupMenu("Model does not have a texture Map%t|Quit")
		return False

	#check if each face uses the same texture map (only one allowed)
	for face in mesh.faces:
		mesh_image=face.image
		if not mesh_image:
			print "Model has a face without a texture Map"
			result=Blender.Draw.PupMenu("Model has a face without a texture Map%t|Quit")
			return False
		if mesh_image!=last_face:
			print "Model has more than 1 texture map assigned"
			result=Blender.Draw.PupMenu("Model has more than 1 texture map assigned%t|Quit")
			return False

	size=mesh_image.getSize()
	#is this really what the user wants
	if (size[0]!=256 or size[1]!=256):
		print "Texture map size is non-standard (not 256x256), it is: ",size[0],"x",size[1]
		result=Blender.Draw.PupMenu("Texture map size is non-standard (not 256x256), it is: "+str(size[0])+"x"+str(size[1])+": Continue?%t|YES|NO")
		if(result==2):
			return False

	#verify frame list data
	user_frame_list=get_frame_list()
	temp=user_frame_list[len(user_frame_list)-1]
	temp_num_frames=temp[2]

	#verify tri/vert/frame counts are within MD2 standard
	face_count=len(mesh.faces)
	vert_count=len(mesh.verts)
	frame_count=temp_num_frames

	if face_count>MD2_MAX_TRIANGLES:
		print "Number of triangles exceeds MD2 standard: ", face_count,">",MD2_MAX_TRIANGLES
		result=Blender.Draw.PupMenu("Number of triangles exceeds MD2 standard: Continue?%t|YES|NO")
		if(result==2):
			return False
	if vert_count>MD2_MAX_VERTICES:
		print "Number of verticies exceeds MD2 standard",vert_count,">",MD2_MAX_VERTICES
		result=Blender.Draw.PupMenu("Number of verticies exceeds MD2 standard: Continue?%t|YES|NO")
		if(result==2):
			return False
	if frame_count>MD2_MAX_FRAMES:
		print "Number of frames exceeds MD2 standard of",frame_count,">",MD2_MAX_FRAMES
		result=Blender.Draw.PupMenu("Number of frames exceeds MD2 standard: Continue?%t|YES|NO")
		if(result==2):
			return False
	#model is OK
	return True

######################################################
# Fill MD2 data structure
######################################################
def fill_md2(md2, object):
	#global defines
	global user_frame_list
	global g_texture_path

	Blender.Window.DrawProgressBar(0.25,"Filling MD2 Data")

	#get a Mesh, not NMesh
	mesh=object.getData(False, True)

	#load up some intermediate data structures
	tex_list={}
	tex_count=0
	#create the vertex list from the first frame
	Blender.Set("curframe", 1)

	#header information
	md2.ident=844121161
	md2.version=8
	md2.num_vertices=len(mesh.verts)
	md2.num_faces=len(mesh.faces)

	#get the skin information
	#use the first faces' image for the texture information
	mesh_image=mesh.faces[0].image
	size=mesh_image.getSize()
	md2.skin_width=size[0]
	md2.skin_height=size[1]
	md2.num_skins=1
	#add a skin node to the md2 data structure
	md2.skins.append(md2_skin())
	md2.skins[0].name=g_texture_path.val+Blender.sys.basename(mesh_image.getFilename())
	if len(md2.skins[0].name)>64:
		print "Texture Path and name is more than 64 characters"
		result=Blender.Draw.PupMenu("Texture path and name is more than 64 characters-Quitting")
		return False

	#put texture information in the md2 structure
	#build UV coord dictionary (prevents double entries-saves space)
	for face in mesh.faces:
		for i in range(0,3):
			t=(face.uv[i])
			tex_key=(t[0],t[1])
			if not tex_list.has_key(tex_key):
				tex_list[tex_key]=tex_count
				tex_count+=1
	md2.num_tex_coords=tex_count #each vert has its own UV coord

	for this_tex in range (0, md2.num_tex_coords):
		md2.tex_coords.append(md2_tex_coord())
	for coord, index in tex_list.iteritems():
		#md2.tex_coords.append(md2_tex_coord())
		md2.tex_coords[index].u=int(coord[0]*md2.skin_width)
		md2.tex_coords[index].v=int((1-coord[1])*md2.skin_height)

	#put faces in the md2 structure
	#for each face in the model
	for this_face in range(0, md2.num_faces):
		md2.faces.append(md2_face())
		for i in range(0,3):
			#blender uses indexed vertexes so this works very well
			md2.faces[this_face].vertex_index[i]=mesh.faces[this_face].verts[i].index
			#lookup texture index in dictionary
			uv_coord=(mesh.faces[this_face].uv[i])
			tex_key=(uv_coord[0],uv_coord[1])
			tex_index=tex_list[tex_key]
			md2.faces[this_face].texture_index[i]=tex_index

	Blender.Window.DrawProgressBar(0.5, "Computing GL Commands")

	#compute GL commands
	md2.num_GL_commands=build_GL_commands(md2)

	#get the frame data
	#calculate 1 frame size  + (1 vert size*num_verts)
	md2.frame_size=40+(md2.num_vertices*4) #in bytes

	#get the frame list
	user_frame_list=get_frame_list()
	if user_frame_list=="default":
		md2.num_frames=198
	else:
		temp=user_frame_list[len(user_frame_list)-1]  #last item
		md2.num_frames=temp[2] #last frame number


	progress=0.5
	progressIncrement=0.25/md2.num_frames

	#fill in each frame with frame info and all the vertex data for that frame
	for frame_counter in range(0,md2.num_frames):

		progress+=progressIncrement
		Blender.Window.DrawProgressBar(progress, "Calculating Frame: "+str(frame_counter))

		#add a frame
		md2.frames.append(md2_frame())
		#update the mesh objects vertex positions for the animation
		Blender.Set("curframe", frame_counter)  #set blender to the correct frame
		mesh.getFromObject(object.name)  #update the mesh to make verts current

#each frame has a scale and transform value that gets the vertex value between 0-255
#since the scale and transform are the same for the all the verts in the frame, we only need
#to figure this out once per frame

		#we need to start with the bounding box
		bounding_box=object.getBoundBox() #uses the object, not the mesh data
		#initialize with the first vertex for both min and max.  X and Y are swapped for MD2 format
		point=bounding_box[0]
		frame_min_x=point[1]
		frame_max_x=point[1]
		frame_min_y=point[0]
		frame_max_y=point[0]
		frame_min_z=point[2]
		frame_max_z=point[2]
		#find min/max values
		for point in bounding_box:
			if frame_min_x>point[1]: frame_min_x=point[1]
			if frame_max_x<point[1]: frame_max_x=point[1]
			if frame_min_y>point[0]: frame_min_y=point[0]
			if frame_max_y<point[0]: frame_max_y=point[0]
			if frame_min_z>point[2]: frame_min_z=point[2]
			if frame_max_z<point[2]: frame_max_z=point[2]

		#the scale is the difference between the min and max (on that axis) / 255
		frame_scale_x=(frame_max_x-frame_min_x)/255
		frame_scale_y=(frame_max_y-frame_min_y)/255
		frame_scale_z=(frame_max_z-frame_min_z)/255

		#translate value of the mesh to center it on the origin
		frame_trans_x=frame_min_x
		frame_trans_y=frame_min_y
		frame_trans_z=frame_min_z

		#fill in the data
		md2.frames[frame_counter].scale=(-frame_scale_x, frame_scale_y, frame_scale_z)
		md2.frames[frame_counter].translate=(-frame_trans_x, frame_trans_y, frame_trans_z)

		#now for the vertices
		for vert_counter in range(0, md2.num_vertices):
			#add a vertex to the md2 structure
			md2.frames[frame_counter].vertices.append(md2_point())
			#figure out the new coords based on scale and transform
			#then translates the point so it's not less than 0
			#then scale it so it's between 0..255
			new_x=int((mesh.verts[vert_counter].co[1]-frame_trans_x)/frame_scale_x)
			new_y=int((mesh.verts[vert_counter].co[0]-frame_trans_y)/frame_scale_y)
			new_z=int((mesh.verts[vert_counter].co[2]-frame_trans_z)/frame_scale_z)
			#put them in the structure
			md2.frames[frame_counter].vertices[vert_counter].vertices=(new_x, new_y, new_z)

			#need to add the lookup table check here
			maxdot = -999999.0;
			maxdotindex = -1;

			for j in range(0,162):
				#dot = (x[0]*y[0]+x[1]*y[1]+x[2]*y[2])
				#swap y and x for difference in axis orientation
				x1=-mesh.verts[vert_counter].no[1]
				y1=mesh.verts[vert_counter].no[0]
				z1=mesh.verts[vert_counter].no[2]
				dot = (x1*MD2_NORMALS[j][0]+
				       y1*MD2_NORMALS[j][1]+
							 z1*MD2_NORMALS[j][2]);
				if (dot > maxdot):
					maxdot = dot;
					maxdotindex = j;

			md2.frames[frame_counter].vertices[vert_counter].lightnormalindex=maxdotindex

			del maxdot, maxdotindex
			del new_x, new_y, new_z
		del frame_max_x, frame_max_y, frame_max_z, frame_min_x, frame_min_y, frame_min_z
		del frame_scale_x, frame_scale_y, frame_scale_z, frame_trans_x, frame_trans_y, frame_trans_z


	#output all the frame names-user_frame_list is loaded during the validation
	for frame_set in user_frame_list:
		for counter in range(frame_set[1]-1, frame_set[2]):
			md2.frames[counter].name=frame_set[0]+"_"+str(counter-frame_set[1]+2)

	#compute these after everthing is loaded into a md2 structure
	header_size=17*4 #17 integers, and each integer is 4 bytes
	skin_size=64*md2.num_skins #64 char per skin * number of skins
	tex_coord_size=4*md2.num_tex_coords #2 short * number of texture coords
	face_size=12*md2.num_faces #3 shorts for vertex index, 3 shorts for tex index
	frames_size=(((12+12+16)+(4*md2.num_vertices)) * md2.num_frames) #frame info+verts per frame*num frames
	GL_command_size=md2.num_GL_commands*4 #each is an int or float, so 4 bytes per

	#fill in the info about offsets
	md2.offset_skins=0+header_size
	md2.offset_tex_coords=md2.offset_skins+skin_size
	md2.offset_faces=md2.offset_tex_coords+tex_coord_size
	md2.offset_frames=md2.offset_faces+face_size
	md2.offset_GL_commands=md2.offset_frames+frames_size
	md2.offset_end=md2.offset_GL_commands+GL_command_size

######################################################
# Get Frame List
######################################################
def get_frame_list():
	global g_frame_filename
	frame_list=[]

	if g_frame_filename.val=="default":
		return MD2_FRAME_NAME_LIST

	else:
	#check for file
		if (Blender.sys.exists(g_frame_filename.val)==1):
			#open file and read it in
			file=open(g_frame_filename.val,"r")
			lines=file.readlines()
			file.close()

			#check header (first line)
			if lines[0].strip()<>"# MD2 Frame Name List":
				print "its not a valid file"
				result=Blender.Draw.PupMenu("This is not a valid frame definition file-using default%t|OK")
				return MD2_FRAME_NAME_LIST
			else:
				#read in the data
				num_frames=0
				for counter in range(1, len(lines)):
					current_line=lines[counter]
					if current_line[0]=="#":
						#found a comment
						pass
					else:
						data=current_line.split()
						frame_list.append([data[0],num_frames+1, num_frames+int(data[1])])
						num_frames+=int(data[1])
				return frame_list
		else:
			print "Cannot find file"
			result=Blender.Draw.PupMenu("Cannot find frame definion file-using default%t|OK")
			return MD2_FRAME_NAME_LIST

######################################################
# Globals for GL command list calculations
######################################################
used_tris=[]
strip_verts=[]
strip_st=[]
strip_tris=[]
strip_count=0

######################################################
# Tri-Strip/Tri-Fan functions
######################################################
def find_strip_length(md2, start_tri, start_vert):
	print "Finding strip length"

	global used
	global strip_verts
	global strip_st
	global strip_tris

	m1=m2=0

	used[start_tri]=2

	strip_verts=[0]*0
	strip_tris=[0]*0
	strip_st=[0]*0


	strip_verts.append(md2.faces[start_tri].vertex_index[start_vert%3])
	strip_verts.append(md2.faces[start_tri].vertex_index[(start_vert+2)%3])
	strip_verts.append(md2.faces[start_tri].vertex_index[(start_vert+1)%3])

	strip_st.append(md2.faces[start_tri].texture_index[start_vert%3])
	strip_st.append(md2.faces[start_tri].texture_index[(start_vert+2)%3])
	strip_st.append(md2.faces[start_tri].texture_index[(start_vert+1)%3])

	strip_count=1
	strip_tris.append(start_tri)

	m1=md2.faces[start_tri].vertex_index[(start_vert+2)%3]
	m2=md2.faces[start_tri].vertex_index[(start_vert+1)%3]

	for tri_counter in range(start_tri+1, md2.num_faces):
		for k in range(0,3):
			if(md2.faces[tri_counter].vertex_index[k]!=m1) or (md2.faces[tri_counter].vertex_index[(k+1)%3]!=m2):
				pass
			else:
				#found a matching edge
				print "Found a triangle with a matching edge: ", tri_counter
				if(used[tri_counter]!=0):
					print "Poop! I can't use it!"
				else:
					print "Yeah! I can use it"

					if(strip_count%2==1):  #is this an odd tri
						print "odd triangle"
						m2=md2.faces[tri_counter].vertex_index[(k+2)%3]
					else:
						print "even triangle"
						m1=md2.faces[tri_counter].vertex_index[(k+2)%3]

					strip_verts.append(md2.faces[tri_counter].vertex_index[(k+2)%3])
					strip_st.append(md2.faces[tri_counter].texture_index[(k+2)%3])
					strip_count+=1
					strip_tris.append(tri_counter)

					used[tri_counter]=2
					tri_counter=start_tri+1 # restart looking

	#clear used counter
	for used_counter in range(0, md2.num_faces):
		if used[used_counter]==2:
			used[used_counter]=0

	return strip_count

#***********************************************
def find_fan_length(md2, start_tri, start_vert):
	print "Finding fan length"

	global used
	global strip_verts
	global strip_tris
	global strip_st

	strip_verts=[0]*0
	strip_tris=[0]*0
	strip_st=[0]*0

	m1=m2=0

	used[start_tri]=2

	strip_verts.append(md2.faces[start_tri].vertex_index[start_vert%3])
	strip_verts.append(md2.faces[start_tri].vertex_index[(start_vert+2)%3])
	strip_verts.append(md2.faces[start_tri].vertex_index[(start_vert+1)%3])

	strip_st.append(md2.faces[start_tri].texture_index[start_vert%3])
	strip_st.append(md2.faces[start_tri].texture_index[(start_vert+2)%3])
	strip_st.append(md2.faces[start_tri].texture_index[(start_vert+1)%3])

	strip_count=1
	strip_tris.append(start_tri)

	m1=md2.faces[start_tri].vertex_index[(start_vert)]
	m2=md2.faces[start_tri].vertex_index[(start_vert+1)%3]

	for tri_counter in range(start_tri+1, md2.num_faces):
		for k in range(0,3):
			if (md2.faces[tri_counter].vertex_index[k]!=m1) or (md2.faces[tri_counter].vertex_index[(k+1)%3]!=m2):
				pass
			else:
				#found a matching edge
				print "Found a triangle with a matching edge: ", tri_counter
				if(used[tri_counter]!=0):
					print "Poop! I can't use it!"
				else:
					print "Yeah! I can use it"

					m2=md2.faces[tri_counter].vertex_index[(k+1)%3]

					strip_verts.append(md2.faces[tri_counter].vertex_index[(k+1)%3])
					strip_st.append(md2.faces[tri_counter].texture_index[(k+1)%3])

					strip_count+=1
					strip_tris.append(tri_counter)

					used[tri_counter]=2
					tri_counter=start_tri+1 #restart looking

	#clear used counter
	for used_counter in range(0, md2.num_faces):
		if used[used_counter]==2:
			used[used_counter]=0

	return strip_count

######################################################
# Build GL command List
######################################################
def build_GL_commands(md2):
	print "Building GL Commands"

	global used
	global strip_verts
	global strip_tris
	global strip_st

	#globals initialization
	used=[0]*md2.num_faces
	print "Used: ", used
	num_commands=0

	for tri_counter in range(0,md2.num_faces):
		if used[tri_counter]!=0:
			print "Found a used triangle: ", tri_counter
		else:
			print "Found an unused triangle: ", tri_counter

			#intialization
			best_length=0
			best_type=0
			best_verts=[0]*0
			best_tris=[0]*0
			best_st=[0]*0


			for start_vert in range(0,3):
				fan_length=find_fan_length(md2, tri_counter, start_vert)
				#fan_length=0
				print "Triangle: ", tri_counter, " Vertex: ", start_vert, " Fan Length: ", fan_length

				if (fan_length>best_length):
					best_type=1
					best_length=fan_length
					best_verts=strip_verts
					best_tris=strip_tris
					best_st=strip_st

				#strip_length=find_strip_length(md2, tri_counter, start_vert)
				strip_length=0
				print "Triangle: ", tri_counter, " Vertex: ", start_vert, " Strip Length: ", strip_length

				if (strip_length>best_length):
					best_type=0
					best_length=strip_length
					best_verts=strip_verts
					best_tris=strip_tris
					best_st=strip_st

			print "Best length for this triangle is: ", best_length
			print "Best type is: ", best_type
			print "Best Tris are: ", best_tris
			print "Best Verts are: ", best_verts
			print "Best ST are: ", best_st


			#mark tris as used
			for used_counter in range(0,best_length):
				used[best_tris[used_counter]]=1

			#create command list
			cmd_list=md2_GL_cmd_list()
			if best_type==0:
				cmd_list.num=(-(best_length+2))
			else:
				cmd_list.num=best_length+2

			num_commands+=1
			for command_counter in range(0,best_length+2):
				cmd=md2_GL_command()
				s=md2.tex_coords[best_st[command_counter]].u
				t=md2.tex_coords[best_st[command_counter]].v
				cmd.s=s/float(md2.skin_width)
				cmd.t=t/float(md2.skin_height)
				cmd.vert_index=best_verts[command_counter]
				num_commands+=3
				cmd_list.cmd_list.append(cmd)
			cmd_list.dump()
			md2.GL_commands.append(cmd_list)

		print "Used: ", used

	#add the null command at the end
	temp_cmdlist=md2_GL_cmd_list()
	temp_cmdlist.num=0
	md2.GL_commands.append(temp_cmdlist)
	num_commands+=1

	#cleanup and return
	used=best_vert=best_st=best_tris=strip_vert=strip_st=strip_tris=0
	return num_commands




######################################################
# Save MD2 Format
######################################################
def save_md2(filename):
	print ""
	print "***********************************"
	print "MD2 Export"
	print "***********************************"
	print ""

	Blender.Window.DrawProgressBar(0.0,"Begining MD2 Export")

	md2=md2_obj()  #blank md2 object to save

	#get the object
	mesh_objs = Blender.Object.GetSelected()

	#check there is a blender object selected
	if len(mesh_objs)==0:
		print "Fatal Error: Must select a mesh to output as MD2"
		print "Found nothing"
		result=Blender.Draw.PupMenu("Must select an object to export%t|OK")
		return

	mesh_obj=mesh_objs[0] #this gets the first object (should be only one)

	#check if it's a mesh object
	if mesh_obj.getType()!="Mesh":
		print "Fatal Error: Must select a mesh to output as MD2"
		print "Found: ", mesh_obj.getType()
		result=Blender.Draw.PupMenu("Selected Object must be a mesh to output as MD2%t|OK")
		return

	ok=validation(mesh_obj)
	if ok==False:
		return

	fill_md2(md2, mesh_obj)
	md2.dump()

	Blender.Window.DrawProgressBar(1.0, "Writing to Disk")

	#actually write it to disk
	file=open(filename,"wb")
	md2.save(file)
	file.close()

	#cleanup
	md2=0

	print "Closed the file"



#! /usr/bin/env python
#
# Following code is distributed under the GNU General Public License version 2
#
'''
This script will find all maps using given texture
Python 2.6 required
'''
import sys, os, fnmatch
import ufomap_io

def check_for_texture(texname, mapname):
    ufo_map = ufomap_io.read_ufo_map(mapname)
    for i in ufo_map:
        for j in i.brushes:
            for k in j.faces:
                if k.texture == texname: #found face with that texture
                    return True
    return False

args = sys.argv

if len(args) != 3:
    print 'Usage: python maps_using_texture.py path texture'
    print ' e.g. python maps_using_texture.py ../../../base/maps/ tex_base/lab_wall001'
    print ' which will process all .map files in given directory and its subdirectories'
    exit()

path = args[1]
texname = args[2]
maps = []

for (folder, subFolders, files) in os.walk(path):
    for filename in files:
        if fnmatch.fnmatch(filename, "*.map"):
            fullpath = os.path.join(folder, filename)
            if check_for_texture(texname, fullpath):
                maps.append(fullpath)

if len(maps) == 0:
    print "This texture is unused"
else:
    for i in maps:
        print i



#! /usr/bin/env python
#
# Following code is distributed under the GNU General Public License version 2
#
'''
A small library to parse and assemble UFO:AI .map files
Python 2.6 required

'''
class Face:
    def __init__(self):
        self.vertexes = [ [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0] ]
        self.texture = "tex_common/nodraw"
        self.offset = [0.0, 0.0]
        self.angle = 0.0
        self.scale = [0.0, 0.0]
        self.cont_flags = 0
        self.surf_flags = 0
        self.surf_value = 0

    def to_string(self): #todo: implement __format__
        verts = "( {0:.8g} {1:.8g} {2:.8g} ) ( {3:.8g} {4:.8g} {5:.8g} ) ( {6:.8g} {7:.8g} {8:.8g} )".format(self.vertexes[0][0], self.vertexes[0][1], self.vertexes[0][2], self.vertexes[1][0], self.vertexes[1][1], self.vertexes[1][2], self.vertexes[2][0], self.vertexes[2][1], self.vertexes[2][2])
        tex = "{0:.8g} {1:.8g} {2:.8g} {3:.8g} {4:.8g}".format(self.offset[0], self.offset[1], self.angle, self.scale[0], self.scale[1])
        flags = "{0} {1} {2}".format(self.cont_flags, self.surf_flags, self.surf_value)
        return " ".join((verts, self.texture, tex, flags))

class Brush:
    def __init__(self):
        self.faces = []

    def to_string(self): #todo: implement __format__
        tmp = ["{"]
        for i in self.faces:
            tmp.append(i.to_string())
        tmp.append("}");
        return "\n".join(tmp)

class Entity:
    def __init__(self):
        self.brushes = []
        self.fields = {}
        self.order = []

    def to_string(self):
        tmp = ["{"]
        fields = self.fields.copy()
        for i in self.order:
            if i in fields:
                tmp.append('"{0}" "{1}"'.format(i, fields[i]))
                del fields[i]
        for i in fields:
            tmp.append('"{0}" "{1}"'.format(i, self.fields[i]))
        brushCount = 0
        for i in self.brushes:
            tmp.append("// brush {0}".format(brushCount))
            tmp.append(i.to_string())
            brushCount += 1
        tmp.append("}")
        return "\n".join(tmp)

def parse_ufo_map(s, verbose = False):
    l = s.splitlines()
    if verbose: print len(l), "lines"

    depth = 0 # 1 - entity, 2 - brush level
    ent = Entity()
    brush = Brush()
    ufo_map = []

    for n, i in enumerate(l):
        tokens = i.partition("//")[0].split() #remove comments and tokenize each line
        count = len(tokens)
        if count == 0: continue #no statements in this line
        elif count == 1:
            if tokens[0] == "{":
                depth += 1
                if depth > 2:
                    print "Syntax error - unexpected { in line", n
                    break
            elif tokens[0] == "}":
                depth -= 1
                if depth == 0: #finalize entity and create new one
                    ufo_map.append(ent)
                    #print "entity", ent.fields.get("classname", "")
                    ent = Entity()
                elif depth == 1: #add brush to entity
                    ent.brushes.append(brush)
                    brush = Brush()
                else:
                    print "Syntax error - unexpected } in line", n
                    break
            else:
                print "Syntax error - can't analyze line", n
                break
        elif depth == 1 and tokens[0][0] == '"': #entity field definition
            tokens = i.split('"') #reparse using " as separator
            if len(tokens) != 5:
                print "Syntax error - can't analyze line", n
                break
            ent.fields[tokens[1]] = tokens[3]
            ent.order.append(tokens[1])
        elif depth == 2 and tokens[0][0] == '(': #brush face definition
            face = Face()
            if count > 13 and tokens[0] == "(" and tokens[4] == ")" and tokens[5] == "(" and tokens[9] == ")" and tokens[10] == "(" and tokens[14] == ")":
                face.vertexes[0] = [float(tokens[1]), float(tokens[2]), float(tokens[3])]
                face.vertexes[1] = [float(tokens[6]), float(tokens[7]), float(tokens[8])]
                face.vertexes[2] = [float(tokens[11]), float(tokens[12]), float(tokens[13])]
            else:
                print "Syntax error - can't analyze line", n
                break
            if count > 15: face.texture = tokens[15]
            if count > 17: face.offset = [ float(tokens[16]), float(tokens[17]) ]
            if count > 18: face.angle = float(tokens[18])
            if count > 20: face.scale = [ float(tokens[19]), float(tokens[20]) ]
            if count > 21: face.cont_flags = int(tokens[21])
            if count > 22: face.surf_flags = int(tokens[22])
            if count > 23: face.surf_value = int(tokens[23])
            brush.faces.append(face)
        else:
            print "Syntax error - can't analyze line", n
            break
    if verbose: print len(ufo_map), "entities read"
    return ufo_map

def ufo_map_to_string(ufo_map):
    tmp = [""] #Radiant output starts with empty line for some reason
    entityCount = 0
    for i in ufo_map:
        tmp.append("// entity {0}".format(entityCount))
        tmp.append(i.to_string())
        entityCount += 1
    tmp.append("\n")
    return "\n".join(tmp)

def read_ufo_map(name, verbose = False):
    f = open(name)
    if verbose: print name
    s = f.read()
    f.close()
    return parse_ufo_map(s, verbose)

def write_ufo_map(ufo_map, name):
    f = open(name, "wb")
    f.write(ufo_map_to_string(ufo_map))
    f.close()



#! /usr/bin/env python
#
# Following code is distributed under the GNU General Public License version 2
#
'''
This script will rescale specific texture used in the given .map(s)
Python 2.6 required
'''
import sys, glob, os, fnmatch
import ufomap_io

def scale_textures(texname, mapname, scalex, scaley):
    changed = False
    ufo_map = ufomap_io.read_ufo_map(mapname, True)
    for i in ufo_map:
        for j in i.brushes:
            for k in j.faces:
                if k.texture == texname: #found face with that texture
                    k.scale[0] *= scalex
                    k.scale[1] *= scaley
                    k.offset[0] /= scalex
                    k.offset[1] /= scaley
                    changed = True

    if changed:
        ufomap_io.write_ufo_map(ufo_map, mapname)
        print "Texture found, map updated"

args = sys.argv
argsCount = len(args)
path = ""
recurse = False
scalex = 1.0
scaley = 1.0
texname = ""

brokenArgs = False
if argsCount<4:
    brokenArgs = True
else:
    if args[2] == "-r":
        recurse = True
        texname = args[3]
        if argsCount == 5:
            scalex = scaley = float(args[4])
        elif argsCount == 6:
            scalex = float(args[4])
            scaley = float(args[5])
        else:
            brokenArgs = True
    else:
        texname = args[2]
        if argsCount == 4:
            scalex = scaley = float(args[3])
        elif argsCount == 5:
            scalex = float(args[3])
            scaley = float(args[4])
        else:
            brokenArgs = True

if brokenArgs:
    print 'Usage: python map_scale_textures.py map_file_name|map_file_wildcard texture scalex scaley'
    print ' e.g. python map_scale_textures.py ../../../base/map/dam.map tex_base/lab_wall001 .5 .33'
    print ' or python map_scale_textures.py ../../../base/map/*.map tex_base/lab_wall001 .5 .33'
    print ' to recurse through subdirectories use: python map_scale_textures.py path -r texture scalex scaley'
    print ' which will process all .map files'
    print 'If only scalex is given, scaley will reuse its value'
    exit()

path = args[1]

if recurse:
    for (folder, subFolders, files) in os.walk(path):
        for filename in files:
            if fnmatch.fnmatch(filename, "*.map"):
                scale_textures(texname, os.path.join(folder, filename), scalex, scaley)
    if path.find("prefabs") < 0:
        print "Don't forget prefabs!"
else:
    #expand wildcards, if any
    names = glob.glob(path)

    for name in names:
        scale_textures(texname, name, scalex, scaley)



#! /usr/bin/env python
#
# Following code is distributed under the GNU General Public License version 2
#
'''
This script will print the list of textures used in the given .map
Python 2.6 required
'''
import sys, glob, os, fnmatch
import ufomap_io

def add_textures(texnames, mapname):
    ufo_map = ufomap_io.read_ufo_map(mapname, True)
    for i in ufo_map:
        for j in i.brushes:
            for k in j.faces:
                texnames[k.texture] = True

args = sys.argv
path = ""
if len(args)<2:
    print 'Usage: python map_list_textures.py map_file_name|map_file_wildcard'
    print ' e.g. python map_list_textures.py ../../../base/map/dam.map'
    print ' or python map_list_textures.py ../../../base/map/*.map'
    print ' to recurse through subdirectories use: python map_list_textures.py path -r'
    print ' which will analyze all .map files'
    print 'Results will be printed to stdout, so add ">result_file_name" (without quotes) if you want to save textures list'
    exit
else:
    path = args[1]

recurse = len(args) == 3 and args[2] == "-r"

texnames = {}

if recurse:
    for (folder, subFolders, files) in os.walk(path):
        for filename in files:
            if fnmatch.fnmatch(filename, "*.map"):
                add_textures(texnames, os.path.join(folder, filename))
else:
    #expand wildcards, if any
    names = glob.glob(path)

    for name in names:
        add_textures(texnames, name)

tmp = texnames.keys()
tmp.sort()
print "\n".join(map(str, tmp))



# ***** BEGIN GPL LICENSE BLOCK *****

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# ***** END GPL LICENCE BLOCK *****

'''
This is script an importer and exporter for the Quake2 "MD2" file format.

The frames are named <frameName><N> with :<br>
 - <N> the frame number<br>
 - <frameName> the name choosen at the last marker (or 'frame' if the last marker has no name or if there is no last marker)

Skins are set using image textures in materials, if it is longer than 63 characters it is truncated.

Thanks to Bob Holcomb for MD2_NORMALS taken from his exporter.
Thanks to David Henry for the documentation about the MD2 file format.
'''

bl_info = {
	"name": "Quake2 MD2 format",
	"description": "Importer and exporter for Quake2 file format (.md2)",
	"author": "DarkRain, based on the work of Bob Holcomb/Sebastian Lieberknecht/Dao Nguyen/Bernd Meyer/Damien Thebault/Erwan Mathieu",
	"version": (1, 0),
	"blender": (2, 63, 0),
	"location": "File > Import/Export > Quake2 MD2",
	"warning": "", # used for warning icon and text in addons panel
	"support": 'COMMUNITY',
	"category": "Import-Export"}

import bpy
from bpy.props import *

from bpy_extras.io_utils import ExportHelper, ImportHelper, unpack_list, unpack_face_list
from bpy_extras.image_utils import load_image

from math import pi
from mathutils import Matrix

import struct
import random
import os
import shutil

MD2_MAX_TRIANGLES = 4096
MD2_MAX_VERTS = 2048
MD2_MAX_FRAMES = 1024		#512 - 1024 is an UFO:AI extension.
MD2_MAX_SKINS = 32
MD2_MAX_SKINNAME = 63
MD2_NORMALS = ((-0.525731, 0.000000, 0.850651),
				(-0.442863, 0.238856, 0.864188),
				(-0.295242, 0.000000, 0.955423),
				(-0.309017, 0.500000, 0.809017),
				(-0.162460, 0.262866, 0.951056),
				( 0.000000, 0.000000, 1.000000),
				( 0.000000, 0.850651, 0.525731),
				(-0.147621, 0.716567, 0.681718),
				( 0.147621, 0.716567, 0.681718),
				( 0.000000, 0.525731, 0.850651),
				( 0.309017, 0.500000, 0.809017),
				( 0.525731, 0.000000, 0.850651),
				( 0.295242, 0.000000, 0.955423),
				( 0.442863, 0.238856, 0.864188),
				( 0.162460, 0.262866, 0.951056),
				(-0.681718, 0.147621, 0.716567),
				(-0.809017, 0.309017, 0.500000),
				(-0.587785, 0.425325, 0.688191),
				(-0.850651, 0.525731, 0.000000),
				(-0.864188, 0.442863, 0.238856),
				(-0.716567, 0.681718, 0.147621),
				(-0.688191, 0.587785, 0.425325),
				(-0.500000, 0.809017, 0.309017),
				(-0.238856, 0.864188, 0.442863),
				(-0.425325, 0.688191, 0.587785),
				(-0.716567, 0.681718,-0.147621),
				(-0.500000, 0.809017,-0.309017),
				(-0.525731, 0.850651, 0.000000),
				( 0.000000, 0.850651,-0.525731),
				(-0.238856, 0.864188,-0.442863),
				( 0.000000, 0.955423,-0.295242),
				(-0.262866, 0.951056,-0.162460),
				( 0.000000, 1.000000, 0.000000),
				( 0.000000, 0.955423, 0.295242),
				(-0.262866, 0.951056, 0.162460),
				( 0.238856, 0.864188, 0.442863),
				( 0.262866, 0.951056, 0.162460),
				( 0.500000, 0.809017, 0.309017),
				( 0.238856, 0.864188,-0.442863),
				( 0.262866, 0.951056,-0.162460),
				( 0.500000, 0.809017,-0.309017),
				( 0.850651, 0.525731, 0.000000),
				( 0.716567, 0.681718, 0.147621),
				( 0.716567, 0.681718,-0.147621),
				( 0.525731, 0.850651, 0.000000),
				( 0.425325, 0.688191, 0.587785),
				( 0.864188, 0.442863, 0.238856),
				( 0.688191, 0.587785, 0.425325),
				( 0.809017, 0.309017, 0.500000),
				( 0.681718, 0.147621, 0.716567),
				( 0.587785, 0.425325, 0.688191),
				( 0.955423, 0.295242, 0.000000),
				( 1.000000, 0.000000, 0.000000),
				( 0.951056, 0.162460, 0.262866),
				( 0.850651,-0.525731, 0.000000),
				( 0.955423,-0.295242, 0.000000),
				( 0.864188,-0.442863, 0.238856),
				( 0.951056,-0.162460, 0.262866),
				( 0.809017,-0.309017, 0.500000),
				( 0.681718,-0.147621, 0.716567),
				( 0.850651, 0.000000, 0.525731),
				( 0.864188, 0.442863,-0.238856),
				( 0.809017, 0.309017,-0.500000),
				( 0.951056, 0.162460,-0.262866),
				( 0.525731, 0.000000,-0.850651),
				( 0.681718, 0.147621,-0.716567),
				( 0.681718,-0.147621,-0.716567),
				( 0.850651, 0.000000,-0.525731),
				( 0.809017,-0.309017,-0.500000),
				( 0.864188,-0.442863,-0.238856),
				( 0.951056,-0.162460,-0.262866),
				( 0.147621, 0.716567,-0.681718),
				( 0.309017, 0.500000,-0.809017),
				( 0.425325, 0.688191,-0.587785),
				( 0.442863, 0.238856,-0.864188),
				( 0.587785, 0.425325,-0.688191),
				( 0.688191, 0.587785,-0.425325),
				(-0.147621, 0.716567,-0.681718),
				(-0.309017, 0.500000,-0.809017),
				( 0.000000, 0.525731,-0.850651),
				(-0.525731, 0.000000,-0.850651),
				(-0.442863, 0.238856,-0.864188),
				(-0.295242, 0.000000,-0.955423),
				(-0.162460, 0.262866,-0.951056),
				( 0.000000, 0.000000,-1.000000),
				( 0.295242, 0.000000,-0.955423),
				( 0.162460, 0.262866,-0.951056),
				(-0.442863,-0.238856,-0.864188),
				(-0.309017,-0.500000,-0.809017),
				(-0.162460,-0.262866,-0.951056),
				( 0.000000,-0.850651,-0.525731),
				(-0.147621,-0.716567,-0.681718),
				( 0.147621,-0.716567,-0.681718),
				( 0.000000,-0.525731,-0.850651),
				( 0.309017,-0.500000,-0.809017),
				( 0.442863,-0.238856,-0.864188),
				( 0.162460,-0.262866,-0.951056),
				( 0.238856,-0.864188,-0.442863),
				( 0.500000,-0.809017,-0.309017),
				( 0.425325,-0.688191,-0.587785),
				( 0.716567,-0.681718,-0.147621),
				( 0.688191,-0.587785,-0.425325),
				( 0.587785,-0.425325,-0.688191),
				( 0.000000,-0.955423,-0.295242),
				( 0.000000,-1.000000, 0.000000),
				( 0.262866,-0.951056,-0.162460),
				( 0.000000,-0.850651, 0.525731),
				( 0.000000,-0.955423, 0.295242),
				( 0.238856,-0.864188, 0.442863),
				( 0.262866,-0.951056, 0.162460),
				( 0.500000,-0.809017, 0.309017),
				( 0.716567,-0.681718, 0.147621),
				( 0.525731,-0.850651, 0.000000),
				(-0.238856,-0.864188,-0.442863),
				(-0.500000,-0.809017,-0.309017),
				(-0.262866,-0.951056,-0.162460),
				(-0.850651,-0.525731, 0.000000),
				(-0.716567,-0.681718,-0.147621),
				(-0.716567,-0.681718, 0.147621),
				(-0.525731,-0.850651, 0.000000),
				(-0.500000,-0.809017, 0.309017),
				(-0.238856,-0.864188, 0.442863),
				(-0.262866,-0.951056, 0.162460),
				(-0.864188,-0.442863, 0.238856),
				(-0.809017,-0.309017, 0.500000),
				(-0.688191,-0.587785, 0.425325),
				(-0.681718,-0.147621, 0.716567),
				(-0.442863,-0.238856, 0.864188),
				(-0.587785,-0.425325, 0.688191),
				(-0.309017,-0.500000, 0.809017),
				(-0.147621,-0.716567, 0.681718),
				(-0.425325,-0.688191, 0.587785),
				(-0.162460,-0.262866, 0.951056),
				( 0.442863,-0.238856, 0.864188),
				( 0.162460,-0.262866, 0.951056),
				( 0.309017,-0.500000, 0.809017),
				( 0.147621,-0.716567, 0.681718),
				( 0.000000,-0.525731, 0.850651),
				( 0.425325,-0.688191, 0.587785),
				( 0.587785,-0.425325, 0.688191),
				( 0.688191,-0.587785, 0.425325),
				(-0.955423, 0.295242, 0.000000),
				(-0.951056, 0.162460, 0.262866),
				(-1.000000, 0.000000, 0.000000),
				(-0.850651, 0.000000, 0.525731),
				(-0.955423,-0.295242, 0.000000),
				(-0.951056,-0.162460, 0.262866),
				(-0.864188, 0.442863,-0.238856),
				(-0.951056, 0.162460,-0.262866),
				(-0.809017, 0.309017,-0.500000),
				(-0.864188,-0.442863,-0.238856),
				(-0.951056,-0.162460,-0.262866),
				(-0.809017,-0.309017,-0.500000),
				(-0.681718, 0.147621,-0.716567),
				(-0.681718,-0.147621,-0.716567),
				(-0.850651, 0.000000,-0.525731),
				(-0.688191, 0.587785,-0.425325),
				(-0.587785, 0.425325,-0.688191),
				(-0.425325, 0.688191,-0.587785),
				(-0.425325,-0.688191,-0.587785),
				(-0.587785,-0.425325,-0.688191),
				(-0.688191,-0.587785,-0.425325))

class MD2:
	def __init__(self, options):
		self.options = options
		self.object = None
		self.ident = 844121161
		self.version = 8
		self.skinWidth = 2 ** 10 - 1 #1023
		self.skinHeight = 2 ** 10 - 1 #1023
		return

	def setObject(self, object):
		self.object = object

	def makeObject(self):
		print("Creating mesh", end='')
		width = max(1.0, float(self.skinWidth))
		height = max(1.0, float(self.skinHeight))
		# Skins
		if self.numSkins > 0:
			material = bpy.data.materials.new(self.name)
			for skin in self.skins:
				skinImg = Util.loadImage(skin, self.name)
				skinTex = bpy.data.textures.new(self.name + skin, type='IMAGE')
				skinTex.image = skinImg
				matTex = material.texture_slots.add()
				matTex.texture = skinTex
				matTex.texture_coords = 'UV'
				matTex.use_map_color_diffuse = True
				matTex.use_map_alpha = True
		print('.', end='')

		# Create the mesh
		mesh = bpy.data.meshes.new(self.name)
		if self.numSkins > 0:
			mesh.materials.append(material)
		mesh.vertices.add(self.numVerts)
		mesh.tessfaces.add(self.numTris)
		mesh.tessface_uv_textures.new()
		print('.', end='')

		# Verts
		mesh.vertices.foreach_set("co", unpack_list(self.frames[0]))
		mesh.transform(Matrix.Rotation(-pi / 2, 4, 'Z'))
		print('.', end='')

		# Tris
		mesh.tessfaces.foreach_set("vertices_raw", unpack_face_list([face[0] for face in self.tris]))
		print('.', end='')

		# UV
		mesh.tessface_uv_textures.active.data.foreach_set("uv_raw", unpack_list([self.uvs[i] for i in unpack_face_list([face[1] for face in self.tris])]))
		if self.numSkins > 0:
			image = mesh.materials[0].texture_slots[0].texture.image
			if image != None:
				for uv in mesh.tessface_uv_textures.active.data:
					uv.image = image
		print('.', end='')

		mesh.validate()
		mesh.update()
		obj = bpy.data.objects.new(mesh.name, mesh)
		base = bpy.context.scene.objects.link(obj)
		bpy.context.scene.objects.active = obj
		base.select = True
		print("Done")

		# Animate
		if self.options.fImportAnimation:
			for i, frame in enumerate(self.frames):
				progressStatus = i / self.numFrames * 100
				#bpy.context.scene.frame_set(i + 1)
				obj.shape_key_add(from_mix=False)
				mesh.vertices.foreach_set("co", unpack_list(frame))
				mesh.transform(Matrix.Rotation(-pi / 2, 4, 'Z'))
				mesh.shape_keys.key_blocks[i].value = 1.0
				mesh.shape_keys.key_blocks[i].keyframe_insert("value", frame=i + 1)
				if i < len(self.frames) - 1:
					mesh.shape_keys.key_blocks[i].value = 0.0
					mesh.shape_keys.key_blocks[i].keyframe_insert("value", frame=i + 2)
				if i > 0:
					mesh.shape_keys.key_blocks[i].value = 0.0
					mesh.shape_keys.key_blocks[i].keyframe_insert("value", frame=i)
				print("Animating - progress: %3i%%\r" % int(progressStatus), end='')
			print("Animating - progress: 100%.")
		print("Model imported")

	def write(self, filePath):
		mesh = self.object.data

		skins = Util.getSkins(mesh, self.options.eTextureNameMethod)
		self.numSkins = len(skins)
		self.numVerts = len(mesh.vertices)
		self.numUV, uvList, uvDict = self.buildTexCoord()
		self.numTris = len(mesh.tessfaces)

		if self.options.fSkipGLCommands:
			self.numGLCmds = 0
		else:
			self.numGLCmds = 1 + self.buildGLcommands()

		self.numFrames = 1
		if self.options.fExportAnimation:
			self.numFrames = 1 + bpy.context.scene.frame_end - bpy.context.scene.frame_start

		self.frameSize = struct.calcsize("<6f16s") + struct.calcsize("<4B") * self.numVerts

		self.ofsSkins = struct.calcsize("<17i")
		self.ofsUV = self.ofsSkins + struct.calcsize("<64s") * self.numSkins
		self.ofsTris = self.ofsUV + struct.calcsize("<2h") * self.numUV
		self.ofsFrames = self.ofsTris + struct.calcsize("<6H") * self.numTris
		self.ofsGLCmds = self.ofsFrames + self.frameSize * self.numFrames
		self.ofsEnd = self.ofsGLCmds + struct.calcsize("<i") * self.numGLCmds

		file = open(filePath, "wb")
		try:
			# write header
			data = struct.pack("<17i",
								self.ident,
								self.version,
								self.skinWidth,
								self.skinHeight,
								self.frameSize,
								self.numSkins,
								self.numVerts,
								self.numUV, #  number of texture coordinates
								self.numTris,
								self.numGLCmds,
								self.numFrames,
								self.ofsSkins,
								self.ofsUV,
								self.ofsTris,
								self.ofsFrames,
								self.ofsGLCmds,
								self.ofsEnd)
			file.write(data)

			# write skin file names
			for iSkin, (skinPath, skinName) in enumerate(skins):
				filePath = bpy.path.abspath(skinPath)

				if self.options.fCopyTextureSxS:
					destPath = os.path.join(os.path.dirname(filePath), os.path.basename(filePath))
					print("Copying texture %s to %s" % (filePath, destPath))
					try:
						shutil.copy(filePath, destPath)
					except:
						print("Copying texture %s to %s failed." % (filePath, destPath))
					if self.options.eTextureNameMethod == 'FILEPATH':
						skinName = destPath

				if len(skinName) > MD2_MAX_SKINNAME:
					print("WARNING: The texture name '%s' is too long. It was automatically truncated." % skinName)
					if self.options.eTextureNameMethod == 'FILEPATH':
						skinName = os.path.basename(filePath)

				data = struct.pack("<64s", bytes(skinName[0:MD2_MAX_SKINNAME], encoding="utf8"))
				file.write(data) # skin name


			for uv in uvList:
				data = struct.pack("<2h",
									int(uv[0] * self.skinWidth),
									int((1 - uv[1]) * self.skinHeight)
									)
				file.write(data) # uv
			del uvList

			for face in mesh.tessfaces:
				# 0,2,1 for good cw/ccw
				data = struct.pack("<3H",
									face.vertices[0],
									face.vertices[2],
									face.vertices[1]
									)
				file.write(data) # vert index
				uvs = mesh.tessface_uv_textures.active.data[face.index].uv
				data = struct.pack("<3H",
									uvDict[(uvs[0][0], uvs[0][1])],
									uvDict[(uvs[2][0], uvs[2][1])],
									uvDict[(uvs[1][0], uvs[1][1])],
									)
				file.write(data) # uv index
			del uvDict

			if self.options.fExportAnimation:
				min = None
				max = None

				timeLineMarkers = []
				for marker in bpy.context.scene.timeline_markers:
					timeLineMarkers.append(marker)

				# sort the markers. The marker with the frame number closest to 0 will be the first marker in the list.
				# The marker with the biggest frame number will be the last marker in the list
				timeLineMarkers.sort(key=lambda marker: marker.frame)
				markerIdx = 0

				# delete markers at same frame positions
				if len(timeLineMarkers) > 1:
					markerFrame = timeLineMarkers[len(timeLineMarkers) - 1].frame
					for i in range(len(timeLineMarkers) - 2, -1, -1):
						if timeLineMarkers[i].frame == markerFrame:
							del timeLineMarkers[i]
						else:
							markerFrame = timeLineMarkers[i].frame

				for frame in range(bpy.context.scene.frame_start, bpy.context.scene.frame_end + 1):
					frameIdx = frame - bpy.context.scene.frame_start + 1
					#Display the progress status of the export in the console
					progressStatus = frameIdx / self.numFrames * 100
					print("Export progress: %3i%%\r" % int(progressStatus), end='')

					bpy.context.scene.frame_set(frame)

					if len(timeLineMarkers) != 0:
						if markerIdx + 1 != len(timeLineMarkers):
							if frame >= timeLineMarkers[markerIdx + 1].frame:
								markerIdx += 1
						name = timeLineMarkers[markerIdx].name
					else:
						name = "frame"

					self.outFrame(file, name + str(frameIdx))
			else:
				self.outFrame(file)

			# gl commands
			if not self.options.fSkipGLCommands:
				for glCmd in self.glCmdList:
					data = struct.pack("<i", glCmd[0])
					file.write(data)
					for cmd in glCmd[1]:
						data = struct.pack("<ffI",
											cmd[0],
											cmd[1],
											cmd[2])
						file.write(data)
				# NULL command
				data = struct.pack("<I", 0)
				file.write(data)

		finally:
			file.close()
		print("Export progress: 100% - Model exported.")

	def read(self, filePath):
		self.filePath = filePath
		self.name = os.path.splitext(os.path.basename(filePath))[0]
		self.skins = []
		self.uvs = [(0.0, 0.0)]
		self.tris = []
		self.frames = []

		print("Reading: %s" % self.filePath, end='')
		progressStatus = 0.0
		inFile = open(self.filePath, "rb")
		try:
			buff = inFile.read(struct.calcsize("<17i"))
			data = struct.unpack("<17i", buff)

			if data[0] != self.ident or data[1] != self.version:
				raise NameError("Invalid MD2 file")

			self.skinWidth = float(max(1, data[2]))
			self.skinHeight = float(max(1, data[3]))
			self.numSkins = data[5]
			self.numVerts = data[6]
			self.numUV = data[7]
			self.numTris = data[8]
			if self.options.fImportAnimation:
				self.numFrames = data[10]
			else:
				self.numFrames = 1
			self.ofsSkins = data[11]
			self.ofsUV = data[12]
			self.ofsTris = data[13]
			self.ofsFrames = data[14]

			# Skins
			if self.numSkins > 0:
				inFile.seek(self.ofsSkins, 0)
				for i in range(self.numSkins):
					buff = inFile.read(struct.calcsize("<64s"))
					data = struct.unpack("<64s", buff)
					self.skins.append(Util.asciiz(data[0].decode("utf-8", "replace")))
			print('.', end='')

			# UV
			inFile.seek(self.ofsUV, 0)
			for i in range(self.numUV):
				buff = inFile.read(struct.calcsize("<2h"))
				data = struct.unpack("<2h", buff)
				self.uvs.append((data[0] / self.skinWidth, 1 - (data[1] / self.skinHeight)))
			print('.', end='')

			# Tris
			inFile.seek(self.ofsTris, 0)
			for i in range(self.numTris):
				buff = inFile.read(struct.calcsize("<6H"))
				data = struct.unpack("<6H", buff)
				self.tris.append(((data[0], data[2], data[1]), (data[3] + 1, data[5] + 1, data[4] + 1)))
			print('.', end='')

			# Frames
			inFile.seek(self.ofsFrames, 0)
			for i in range(self.numFrames):
				buff = inFile.read(struct.calcsize("<6f16s"))
				data = struct.unpack("<6f16s", buff)
				verts = []
				for j in range(self.numVerts):
					buff = inFile.read(struct.calcsize("<4B"))
					vert = struct.unpack("<4B", buff)
					verts.append((data[0] * vert[0] + data[3], data[1] * vert[1] + data[4], data[2] * vert[2] + data[5]))
				self.frames.append(verts)
			print('.', end='')
		finally:
			inFile.close()
		print("Done")

	def outFrame(self, file, frameName = "frame"):
		mesh = self.object.to_mesh(bpy.context.scene, True, 'PREVIEW')

		mesh.transform(self.object.matrix_world)
		mesh.transform(Matrix.Rotation(pi / 2, 4, 'Z'))

		###### compute the bounding box ###############
		min = [mesh.vertices[0].co[0],
			   mesh.vertices[0].co[1],
			   mesh.vertices[0].co[2]]
		max = [mesh.vertices[0].co[0],
			   mesh.vertices[0].co[1],
			   mesh.vertices[0].co[2]]

		for vert in mesh.vertices:
			for i in range(3):
				if vert.co[i] < min[i]:
					min[i] = vert.co[i]
				if vert.co[i] > max[i]:
					max[i] = vert.co[i]
		########################################

		# BL: some caching to speed it up:
		# -> sd_ gets the vertices between [0 and 255]
		#    which is our important quantization.
		sdx = (max[0] - min[0]) / 255.0
		sdy = (max[1] - min[1]) / 255.0
		sdz = (max[2] - min[2]) / 255.0
		isdx = 255.0 / (max[0] - min[0])
		isdy = 255.0 / (max[1] - min[1])
		isdz = 255.0 / (max[2] - min[2])

		# note about the scale: self.object.scale is already applied via matrix_world
		data = struct.pack("<6f16s",
							# writing the scale of the model
							sdx,
							sdy,
							sdz,
							## now the initial offset (= min of bounding box)
							min[0],
							min[1],
							min[2],
							# and finally the name.
							bytes(frameName, encoding="utf8"))

		file.write(data) # frame header

		for vert in mesh.vertices:
			# find the closest normal for every vertex
			for iN in range(162):
				dot = vert.normal[1] * MD2_NORMALS[iN][0] + \
					-vert.normal[0] * MD2_NORMALS[iN][1] + \
					vert.normal[2] * MD2_NORMALS[iN][2]

				if iN == 0 or dot > maxDot:
					maxDot = dot
					bestNormalIndex = iN

			# and now write the normal.
			data = struct.pack("<4B",
								int((vert.co[0] - min[0]) * isdx),
								int((vert.co[1] - min[1]) * isdy),
								int((vert.co[2] - min[2]) * isdz),
								bestNormalIndex)

			file.write(data) # write vertex and normal

	def buildTexCoord(self):
		# Create an UV coord dictionary to avoid duplicate entries and save space
		meshTextureFaces = self.object.data.tessface_uv_textures.active.data
		uvList = []
		uvDict = {}
		uvCount = 0
		for uv in [(uvs[0], uvs[1]) for meshTextureFace in meshTextureFaces for uvs in meshTextureFace.uv]:
			if uv not in uvDict.keys():
				uvList.append(uv)
				uvDict[uv] = uvCount
				uvCount += 1

		return uvCount, uvList, uvDict

	def findStripLength(self, startTri, startVert):
		mesh = self.object.data
		meshTextureFaces = mesh.tessface_uv_textures.active.data
		numFaces = len(mesh.tessfaces)

		self.cmdVerts = []
		self.cmdTris = []
		self.cmdUV = []
		self.used[startTri] = 2

		self.cmdVerts.append(mesh.tessfaces[startTri].vertices_raw[startVert % 3])
		self.cmdVerts.append(mesh.tessfaces[startTri].vertices_raw[(startVert + 2) % 3])
		self.cmdVerts.append(mesh.tessfaces[startTri].vertices_raw[(startVert + 1) % 3])
		self.cmdUV.append(meshTextureFaces[startTri].uv[startVert %3])
		self.cmdUV.append(meshTextureFaces[startTri].uv[(startVert + 2) % 3])
		self.cmdUV.append(meshTextureFaces[startTri].uv[(startVert + 1) % 3])

		stripCount = 1
		self.cmdTris.append(startTri)
		m1 = mesh.tessfaces[startTri].vertices_raw[(startVert + 2) % 3]
		m2 = mesh.tessfaces[startTri].vertices_raw[(startVert + 1) % 3]

		for triCounter in range(startTri + 1, numFaces):
			for k in range(3):
				if(mesh.tessfaces[triCounter].vertices_raw[k] == m1) and (mesh.tessfaces[triCounter].vertices_raw[(k + 1) % 3] == m2):
					if(self.used[triCounter] == 0):
						if(stripCount % 2 == 1):  #is this an odd tri
							m1 = mesh.tessfaces[triCounter].vertices_raw[(k + 2) % 3]
						else:
							m2=mesh.tessfaces[triCounter].vertices_raw[(k + 2) % 3]

						self.cmdVerts.append(mesh.tessfaces[triCounter].vertices_raw[(k + 2) % 3])
						self.cmdUV.append(meshTextureFaces[triCounter].uv[(k + 2) % 3])
						stripCount += 1
						self.cmdTris.append(triCounter)

						self.used[triCounter] = 2
						triCounter = startTri + 1 # restart looking

		#clear used counter
		for usedCounter in range(numFaces):
			if self.used[usedCounter] == 2:
				self.used[usedCounter] = 0

		return stripCount

	def findFanLength(self, startTri, startVert):
		mesh = self.object.data
		meshTextureFaces = mesh.tessface_uv_textures.active.data
		numFaces = len(mesh.tessfaces)

		self.cmdVerts = []
		self.cmdTris = []
		self.cmdUV = []
		self.used[startTri] = 2

		self.cmdVerts.append(mesh.tessfaces[startTri].vertices_raw[startVert % 3])
		self.cmdVerts.append(mesh.tessfaces[startTri].vertices_raw[(startVert + 2) % 3])
		self.cmdVerts.append(mesh.tessfaces[startTri].vertices_raw[(startVert + 1) % 3])
		self.cmdUV.append(meshTextureFaces[startTri].uv[startVert % 3])
		self.cmdUV.append(meshTextureFaces[startTri].uv[(startVert + 2) % 3])
		self.cmdUV.append(meshTextureFaces[startTri].uv[(startVert + 1) % 3])

		fanCount = 1
		self.cmdTris.append(startTri)
		m2 = mesh.tessfaces[startTri].vertices_raw[(startVert) % 3]
		m1 = mesh.tessfaces[startTri].vertices_raw[(startVert + 1) % 3]

		for triCounter in range(startTri + 1, numFaces):
			for k in range(3):
				if (mesh.tessfaces[triCounter].vertices_raw[k] == m1) and (mesh.tessfaces[triCounter].vertices_raw[(k + 1) % 3] == m2):
					if(self.used[triCounter] == 0):
						m1 = mesh.tessfaces[triCounter].vertices_raw[(k + 2) % 3]
						self.cmdVerts.append(mesh.tessfaces[triCounter].vertices_raw[(k + 2) % 3])
						self.cmdUV.append(meshTextureFaces[triCounter].uv[(k + 2) % 3])
						fanCount += 1
						self.cmdTris.append(triCounter)

						self.used[triCounter] = 2
						triCounter=startTri + 1 #restart looking

		#clear used counter
		for usedCounter in range(numFaces):
			if self.used[usedCounter] == 2:
				self.used[usedCounter] = 0

		return fanCount

	def buildGLcommands(self):
		numFaces = len(self.object.data.tessfaces)
		self.used = [0] * numFaces
		numCommands = 0
		self.glCmdList = []

		for triCounter in range(numFaces):
			if self.used[triCounter] == 0:
				#intialization
				bestLength = 0
				bestType = 0
				bestVerts = []
				bestTris = []
				bestUV = []

				for startVert in range(3):
					cmdLength = self.findFanLength(triCounter, startVert)
					if (cmdLength > bestLength):
						bestType = 1
						bestLength = cmdLength
						bestVerts = self.cmdVerts
						bestTris = self.cmdTris
						bestUV = self.cmdUV

					cmdLength = self.findStripLength(triCounter, startVert)
					if (cmdLength > bestLength):
						bestType = 0
						bestLength = cmdLength
						bestVerts = self.cmdVerts
						bestTris = self.cmdTris
						bestUV = self.cmdUV

				#mark tris as used
				for usedCounter in range(bestLength):
					self.used[bestTris[usedCounter]] = 1

				cmd = []
				if bestType == 0:
					num = bestLength + 2
				else:
					num = (-(bestLength + 2))

				numCommands += 1
				for cmdCounter in range(bestLength + 2):
					# (u,v) in blender -> (u,1-v)
					cmd.append((bestUV[cmdCounter][0], 1.0-bestUV[cmdCounter][1], bestVerts[cmdCounter]))
					numCommands += 3

				self.glCmdList.append((num, cmd))

		del self.used, bestVerts, bestUV, bestTris, self.cmdVerts, self.cmdUV, self.cmdTris
		return numCommands

class Util:
	@staticmethod
	def pickName():
		name = "_MD2Obj_" + str(random.random())
		return name[0:20]

	# deletes an object from Blender (remove + unlink)
	@staticmethod
	def deleteObject(object):
		bpy.context.scene.objects.unlink(object)
		bpy.data.objects.remove(object)

	# duplicates the given object and returns it
	@staticmethod
	def duplicateObject(object, name):
		# backup the current object selection and current active object
		selObjects = bpy.context.selected_objects[:]
		actObject = bpy.context.active_object

		# deselect all selected objects
		bpy.ops.object.select_all(action='DESELECT')
		# select the object which we want to duplicate
		object.select = True

		# duplicate the selected object
		bpy.ops.object.duplicate()

		# the duplicated object is automatically selected
		copyObj = bpy.context.selected_objects[0]

		# rename the object with the given name
		copyObj.name = name

		# select all objects which have been previously selected and make active the previous active object
		bpy.context.scene.objects.active = actObject
		for obj in selObjects:
			obj.select = True

		return copyObj

	@staticmethod
	def applyModifiers(object, possibleNewName):
		# backup the current object selection and current active object
		selObjects = bpy.context.selected_objects[:]
		actObject = bpy.context.active_object

		# deselect all selected objects
		bpy.ops.object.select_all(action='DESELECT')
		# select the object which we want to apply modifiers to
		object.select = True

		# duplicate the selected object
		bpy.ops.object.duplicate()

		# the duplicated object is automatically selected
		modifiedObj = bpy.context.selected_objects[0]

		# now apply all modifiers except the Armature modifier...
		for modifier in modifiedObj.modifiers:
			if modifier.type == 'ARMATURE':
				# these must stay for the animation
				continue

			# all others can be applied.
			bpy.ops.object.modifier_apply(modifier=modifier.name)

		# select all objects which have been previously selected and make active the previous active object
		bpy.context.scene.objects.active = actObject
		for obj in selObjects:
			obj.select = True

		if modifiedObj.name == object.name:
			# no modifier applied.
			return object

		# modifiers were applied:
		modifiedObj.name = possibleNewName

		return modifiedObj

	# returns the mesh of the object and return object.data (mesh)
	@staticmethod
	def triangulateMesh(object):
		mesh = object.data

		# make the object the active object!
		bpy.context.scene.objects.active = object

		try:
			bpy.ops.object.modifier_add(type='TRIANGULATE')
			for i, modifier in enumerate(object.modifiers):
				if modifier.type == 'TRIANGULATE':
					for _ in range(i):
						bpy.ops.object.modifier_move_up(modifier=modifier.name)
					bpy.ops.object.modifier_apply(modifier=modifier.name)
					break;
		except:
			bpy.ops.object.mode_set(mode='EDIT', toggle=False)
			bpy.ops.mesh.select_all(action='SELECT')
			# Beauty causes Blender 2.66 to freeze sometimes
			bpy.ops.mesh.quads_convert_to_tris(use_beauty=False)
			bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

		mesh.update(calc_tessface=True)
		return mesh

	@staticmethod
	def getSkins(mesh, method):
		skins = []
		for material in mesh.materials:
			for texSlot in material.texture_slots:
				if not texSlot or texSlot.texture.type != 'IMAGE':
					continue
				if method == 'BASENAME':
					texname = os.path.basename(texSlot.texture.image.filepath)
				elif method == 'FILEPATH':
					texname = texSlot.texture.image.filepath
				else:
					texname = texSlot.texture.image.name
				skins.append((texSlot.texture.image.filepath, texname))

		return skins

	@staticmethod
	def loadImage(imagePath, filePath):
		image = load_image(imagePath, os.path.dirname(filePath), recursive=False)
		if image is not None:
			return image
		return load_image(imagePath, os.path.dirname(filePath), recursive=False, place_holder=True)

	@staticmethod
	def asciiz(s):
		for i, c in enumerate(s):
			if ord(c) == 0:
				return s[:i]

class ObjectInfo:
	def __init__(self, object):
		self.triang = True
		self.vertices = -1
		self.faces = 0
		self.status = ('','')
		self.frames = 1 + bpy.context.scene.frame_end - bpy.context.scene.frame_start
		self.isMesh = object and object.type == 'MESH'

		if self.isMesh:
			originalObject = object
			mesh = object.data

			self.skins = Util.getSkins(mesh, 'DATANAME')

			tmpObjectName = Util.pickName()
			try:
				# apply the modifiers
				object = Util.applyModifiers(object, tmpObjectName)

				if self.triang:
					if object.name != tmpObjectName: # not yet copied: do it now.
						object = Util.duplicateObject(object, tmpObjectName)
					mesh = Util.triangulateMesh(object)

				self.status = (str(len(mesh.vertices)) + " vertices", str(len(mesh.tessfaces)) + " faces")
				self.faces = len(mesh.tessfaces)
				self.vertices = len(mesh.vertices)
				self.isUnwrapped = len(mesh.tessface_uv_textures) > 0

			finally:
				if object.name == tmpObjectName:
					originalObject.select = True
					bpy.context.scene.objects.active = originalObject
					Util.deleteObject(object)
		print(originalObject.name + ": ", self.status)

class Export_MD2(bpy.types.Operator, ExportHelper):
	'''Export selection to Quake2 file format (.md2)'''
	bl_idname = "export_quake.md2"
	bl_label = "Export selection to Quake2 file format (.md2)"
	filename_ext = ".md2"

	fSkipGLCommands = BoolProperty(name="Skip GL Commands",
									description="Skip GL Commands to reduce file size",
									default=True)

	fExportAnimation = BoolProperty(name="Export animation",
									description="Export all frames",
									default=False)

	typesExportSkinNames = [
							('DATANAME', "Data block name", "Use image datablock names"),
							('FILEPATH', "File path", "Use image file paths"),
							('BASENAME', "File name", "Use image file names"),
							]
	eTextureNameMethod = EnumProperty(name="Skin names",
										description="Choose skin naming method",
										items=typesExportSkinNames,)

	fCopyTextureSxS = BoolProperty(name="Copy texture(s) next to .md2",
									description="Try to copy testures to md2 directory (won't overwrite files)",
									default=False)

	def __init__(self):
		try:
			self.object = bpy.context.selected_objects[0]
		except:
			self.object = None

		# go into object mode before we start the actual export procedure
		bpy.ops.object.mode_set(mode='OBJECT', toggle=False)

		self.info = ObjectInfo(self.object)

	def execute(self, context):

		props = self.properties
		filePath = self.filepath
		filePath = bpy.path.ensure_ext(filePath, self.filename_ext)

		object = self.object
		originalObject = object

		if self.info.frames > MD2_MAX_FRAMES and self.fExportAnimation:
			raise NameError("There are too many frames (%i), at most %i are supported in md2" % (info.frames, MD2_MAX_FRAMES))

		# different name each time or we can't unlink it later
		tmpObjectName = Util.pickName()

		object = Util.applyModifiers(object, tmpObjectName)

		if self.info.triang:
			if object.name != tmpObjectName: # not yet copied: do it now.
				object = Util.duplicateObject(object, tmpObjectName)
			mesh = Util.triangulateMesh(object)

		# save the current frame to reset it after export
		if self.fExportAnimation:
			frame = bpy.context.scene.frame_current

		try:
			md2 = MD2(self)
			md2.setObject(object)
			md2.write(filePath)
		finally:
			if object.name == tmpObjectName:
				originalObject.select = True
				bpy.context.scene.objects.active = originalObject
				Util.deleteObject(object)
			if self.fExportAnimation:
				bpy.context.scene.frame_set(frame)

		self.report({'INFO'},  "Model '%s' exported" % originalObject.name)
		return {'FINISHED'}

	def invoke(self, context, event):
		if not context.selected_objects:
			self.report({'ERROR'}, "Please, select an object to export!")
			return {'CANCELLED'}

		# check constrains
		if len(bpy.context.selected_objects) > 1:
			self.report({'ERROR'}, "Please, select exactly one object to export!")
			return {'CANCELLED'}
		if not self.info.isMesh:
			self.report({'ERROR'}, "Only meshes can be exported (selected object is of type '%s')" % (self.object.type))
			return {'CANCELLED'}
		if self.info.faces > MD2_MAX_TRIANGLES:
			self.report({'ERROR'},
				"Object has too many (triangulated) faces (%i), at most %i are supported in md2" % (self.info.faces, MD2_MAX_TRIANGLES))
			return {'CANCELLED'}
		if self.info.vertices > MD2_MAX_VERTS:
			self.report({'ERROR'},
				"Object has too many vertices (%i), at most %i are supported in md2" % (self.info.vertices, MD2_MAX_VERTS))
			return {'CANCELLED'}
		if not self.info.isUnwrapped:
			self.report({'ERROR'}, "Mesh must be unwrapped")
			return {'CANCELLED'}

		wm = context.window_manager
		wm.fileselect_add(self)
		return {'RUNNING_MODAL'}

class Import_MD2(bpy.types.Operator, ImportHelper):
	'''Import Quake2 format file (md2)'''
	bl_idname = "import_quake.md2"
	bl_label = "Import Quake2 format mesh (md2)"

	fImportAnimation = BoolProperty(name="Import animation",
									description="Import all frames",
									default=True)

	def __init__(self):
		if len(bpy.context.selected_objects) > 0:
			# go into object mode before we start the actual import procedure
			bpy.ops.object.mode_set(mode='OBJECT', toggle = False)

	def execute(self, context):
		# deselect all objects
		bpy.ops.object.select_all(action='DESELECT')

		md2 = MD2(self)
		md2.read(self.filepath)
		md2.makeObject()

		bpy.context.scene.update()
		self.report({'INFO'},  "File '%s' imported" % self.filepath)
		return {'FINISHED'}

def menu_func_export(self, context):
	self.layout.operator(Export_MD2.bl_idname, text="Quake II's MD2 (.md2)")

def menu_func_import(self, context):
	self.layout.operator(Import_MD2.bl_idname, text="Quake II's MD2 (.md2)")

def register():
	bpy.utils.register_module(__name__)
	bpy.types.INFO_MT_file_export.append(menu_func_export)
	bpy.types.INFO_MT_file_import.append(menu_func_import)

def unregister():
	bpy.utils.unregister_module(__name__)
	bpy.types.INFO_MT_file_export.remove(menu_func_export)
	bpy.types.INFO_MT_file_import.remove(menu_func_import)

if __name__ == "__main__":
	register()



#!BPY

"""
Name: 'MD2 tags (.tag)'
Blender: 244
Group: 'Export'
Tooltip: 'Export to Quake2 tag file format for UFO:AI (.tag).'
"""

__author__ = 'Werner Hoehrer'
__version__ = '0.0.13'
__url__ = ["UFO: Alien Invasion, http://ufoai.sourceforge.net",
     "Support forum, http://ufoai.org/forum/", "blender", "ufoai"]
__email__ = ["Werner Hoehrer, bill_spam2:yahoo*de", "scripts"]
__bpydoc__ = """\
This script exports a Quake 2 tag file as used in UFO:AI (MD2 TAG).

This file format equals the _parts_ of the MD3 format used for animated tags, but nothing more. i.e tagnames+positions

Base code taken from the md2 exporter by Bob Holcomb. Many thanks.
"""

# ***** BEGIN GPL LICENSE BLOCK *****
#
# Script copyright (C): Werner Hoehrer
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------
# Changelog
# 	0.0.13	Destructavator fixed off-by-one frame bug
# 	0.0.12	Hoehrer	matrix transformation for rotation matrix seems to be ok now. Importing still needs to be fixed/synced though!
# 	0.0.11	Hoehrer	Some cleanup, updated status messages + played around a bit to get tag-rotation to work (with no success yet)
# 	0.0.10	Hoehrer	changed getLocation() to getLocation('worldspace').
# --------------------------------------------------------------------------

import Blender
from Blender import *
from Blender.Draw import *
from Blender.BGL import *
from Blender.Window import *
from Blender.Object import *
from Blender.Mathutils import *

import math
import struct, string
from types import *

######################################################
# BEGIN ----------------------------------------------
# Additional functions used by Ex- and Importer
######################################################

######################################################
# Returns the string from a null terminated string.
######################################################
def asciiz (s):
	n = 0
	while (ord(s[n]) != 0):
		n = n + 1
	return s[0:n]

######################################################
# Apply a matrix to a vert and return a tuple.
#	http://en.wikibooks.org/wiki/Blender_3D:_Blending_Into_Python/Cookbook#Apply_Matrix
######################################################
#def apply_transform(verts, matrix):
#	x, y, z = verts
#	return\
#	x*matrix[0][0] + y*matrix[1][0] + z*matrix[2][0] + matrix[3][0],\
#	x*matrix[0][1] + y*matrix[1][1] + z*matrix[2][1] + matrix[3][1],\
#	x*matrix[0][2] + y*matrix[1][2] + z*matrix[2][2] + matrix[3][2]

######################################################
# Get Angle between 3 points
#	Get the angle between line AB and BC where B is the elbo.
#	http://en.wikibooks.org/wiki/Blender_3D:_Blending_Into_Python/Cookbook#Get_Angle_between_3_points
#	Uses Blender.Mathutils.AngleBetweenVecs and Blender.Mathutils.Vector
######################################################
def getAng3pt3d(avec, bvec, cvec):
	avec, bvec, cvec = Vector(avec), Vector(bvec), Vector(cvec)

	try:
		ang = AngleBetweenVecs(avec - bvec,  cvec - bvec)
		if ang != ang:
			raise  "ERROR angle between Vecs"
		else:
			return ang
	except:
		print '\tAngleBetweenVecs failed, zero length?'
		return 0

######################################################
# Convert the three axes and the location into a transform-matrix. (dunno why the location is used at all?)
#
# Md3 tag matrix transform code (see also load_md2_tags) is balantly stolen from blender md3 importer :)
# Many thanks to PhaethonH, Bob Holcomb, Robert (Tr3B) Beckebans and further (www.gametutorials.com) DigiBen! and PhaethonH
######################################################
def MatrixSetupTransform(forward, left, up, origin):
	return [
		[forward[0],	forward[1],	forward[2],	origin[0]],
		[left[0],	left[1],	left[2],	origin[1]],
		[up[0],	up[1],	up[2],	origin[2]],
		[0.0,	0.0,		0.0,		1.0]
	]


######################################################
# Additional functions used by Ex- and Importer
# END ------------------------------------------------
######################################################

######################################################
# GUI Loader
######################################################

# Export globals
g_filename		= Create("default.tag") # default value is set here.
g_filename_search	= Create("")
g_scale			= Create(1.0)		# default value is set here.

# Events
EVENT_NOEVENT=1
EVENT_CHOOSE_FILENAME=2
EVENT_SAVE_MD2TAGS=3
EVENT_LOAD_MD2TAGS=4
EVENT_EXIT=100

######################################################
# MD2 TAG Constants
######################################################
MD2_MAX_TAGS=4096
MD2_MAX_FRAMES=512

######################################################
# Callbacks for Window functions
######################################################
def filename_callback(input_filename):
	global g_filename
	g_filename.val=input_filename

def draw_gui():
	global g_scale
	global g_filename
	global EVENT_NOEVENT,EVENT_SAVE_MD2TAGS,EVENT_CHOOSE_FILENAME,EVENT_EXIT, EVENT_LOAD_MD2TAGS

	########## Titles
	glClear(GL_COLOR_BUFFER_BIT)
	glRasterPos2d(10, 120)
	Text("MD2 TAGs Export & Import")

	######### Parameters GUI Buttons
	######### MD2 Filename text entry
	left = 10
	width = 200
	g_filename = String("File: ", EVENT_NOEVENT, left, 95, width-left, 18,
			g_filename.val, 255, "MD2 TAGs file to load/save (Import/Export).")
	########## MD2 TAGs File Search Button
	Button("Search",EVENT_CHOOSE_FILENAME, width, 95, 50, 18)

	########## Scale slider-default is 1/8 which is a good scale for md2->blender
	g_scale= Slider("Scale: ", EVENT_NOEVENT,
			left, 75, width-left, 18,
			g_scale.val, 0.001, 100.0, 0, "Scale factor for MD2 TAGs (Import/Export)");

	g_frames_info = Label('Frames: ' + str(1 + Blender.Get('endframe') - Blender.Get('staframe')),
			left, 55, width-left, 20)

	number_of_tags = num_tags()
	if (number_of_tags > 0):
		objects_info = 'Tags: ' + str(number_of_tags)
	else:
		objects_info = 'No tags (Empties) selected!'

	g_objects_info = Label(objects_info,
			left, 35, width-left, 20)

	######### Draw and Exit Buttons
	Button("Export",EVENT_SAVE_MD2TAGS , 10, 10, 80, 18)
	Button("Import",EVENT_LOAD_MD2TAGS , 90, 10, 80, 18)
	Button("Exit",EVENT_EXIT , 170, 10, 80, 18)

def event(evt, val):
	if (evt == QKEY and not val):
		Exit()

def bevent(evt):
	global g_filename
	global EVENT_NOEVENT,EVENT_SAVE_MD2TAGS,EVENT_EXIT,EVENT_LOAD_MD2TAGS

	######### Manages GUI events
	if (evt==EVENT_EXIT):
		Blender.Draw.Exit()
	elif (evt==EVENT_CHOOSE_FILENAME):
		FileSelector(filename_callback, "MD2 TAG File Selection")
	elif (evt==EVENT_SAVE_MD2TAGS):
		if (g_filename.val == "model"):
			save_md2_tags("blender.tag")
			Blender.Draw.Exit()
			return
		else:
			save_md2_tags(g_filename.val)
			Blender.Draw.Exit()
			return
	elif (evt==EVENT_LOAD_MD2TAGS):
		if (g_filename.val == "model"):
			load_md2_tags("blender.tag")
			Blender.Draw.Exit()
			return
		else:
			load_md2_tags(g_filename.val)
			Blender.Draw.Exit()
			return

Register(draw_gui, event, bevent)


######################################################
# MD2 TAG data structures
######################################################
class md2_tagname:
	name=""
	binary_format="<64s"
	def __init__(self, string):
		self.name=string
	def getSize(self):
		return struct.calcsize(self.binary_format)
	def save(self, file):
		temp_data=self.name
		data=struct.pack(self.binary_format, temp_data)
		file.write(data)
	def load(self, file):
		temp_data = file.read(self.getSize())
		data=struct.unpack(self.binary_format, temp_data)
		self.name=asciiz(data[0])
		return self
	def dump (self):
		print "MD2 tagname"
		print "tag: ",self.name
		print ""

class md2_tag:
	####################################
	# Save-function in MD2 exporter:
	#	x1=-mesh.verts[vert_counter].no[1]
	#	y1=mesh.verts[vert_counter].no[0]
	#	z1=mesh.verts[vert_counter].no[2]
	# => md2file(x,y,z) = blender(-y,x,z)
	# Load-function in MD2 importer:
	#	mesh.verts[j].co[0]=y
	#	mesh.verts[j].co[1]=-x
	#	mesh.verts[j].co[2]=z
	# => blender(x,y,z) = md2file(y,-x,z)
	####################################
	global g_scale
	axis1	= []
	axis2	= []
	axis3	= []
	origin	= []

	binary_format="<12f"	#little-endian (<), 12 floats (12f)	| See http://docs.python.org/lib/module-struct.html for more info.


	def __init__(self):
		origin	= Mathutils.Vector(0.0, 0.0, 0.0)
		axis1	= Mathutils.Vector(0.0, 0.0, 0.0)
		axis2	= Mathutils.Vector(0.0, 0.0, 0.0)
		axis3	= Mathutils.Vector(0.0, 0.0, 0.0)

	def getSize(self):
		return struct.calcsize(self.binary_format)

	# Function + general data-structure stolen from https://intranet.dcc.ufba.br/svn/indigente/trunk/scripts/md3_top.py
	# Kudos to Bob Holcomb and Damien McGinnes
	def foo(self, vet):
		for i in xrange(0,3):
			vet[i] = float(vet[i])
		return vet

	def save(self, file):
		# Prepare temp data for export with transformed axes (matrix transform)
		temp_data=(
		# rotation matrix
		float(self.axis2[1]), -float(self.axis2[0]), -float(self.axis2[2]),
		float(-self.axis1[1]), float(self.axis1[0]), float(self.axis1[2]),
		float(-self.axis3[1]), float(self.axis3[0]), float(self.axis3[2]),
		# position
		float(-self.origin[1]), float(self.origin[0]), float(self.origin[2])
		)

		# Apply scale to tempdata if it was set in the dialog.
		if (g_scale.val != 1.0):
			temp_data[9] = temp_data[9] * g_scale.val
			temp_data[10] = temp_data[10] * g_scale.val
			temp_data[11] = temp_data[11] * g_scale.val

		# Prepare serialised data for writing.
		data=struct.pack(self.binary_format,
			temp_data[0], temp_data[1], temp_data[2],
			temp_data[3], temp_data[4], temp_data[5],
			temp_data[6], temp_data[7], temp_data[8],
			temp_data[9], temp_data[10], temp_data[11])

		# Write it
		file.write(data)

	def load(self, file):
		# Read data from file with the defined size.
		temp_data = file.read(self.getSize())

		# De-serialize the data.
		data = struct.unpack(self.binary_format, temp_data)

		# Parse data from file into pythin md2 structure
		self.axis1	= (data[0], data[1], data[2])
		self.axis2	= (data[3], data[4], data[5])
		self.axis3	= (data[6], data[7], data[8])
		self.origin	= (data[9], data[10], data[11])

		# Convert the orientation of the axes to the blender format.
		# TODO: fix transform to match the one for saving!
		self.origin = (self.origin[1], -self.origin[0], self.origin[2])
		#self.axis1 = (self.axis1[1], -self.axis1[0], self.axis1[2])
		#self.axis2 = (self.axis2[1], -self.axis2[0], self.axis2[2])
		#self.axis3 = (self.axis3[1], -self.axis3[0], self.axis3[2])

		# Apply scale to imported data if it was set in the dialog.
		if (g_scale.val != 1.0):
			self.origin = (
				self.origin[0] * g_scale.val,
				self.origin[1] * g_scale.val,
				self.origin[2] * g_scale.val
				)

		return self

	def dump(self):
		print "MD2 Point Structure"
		print "origin: ", self.origin[0], " ",self.origin[1] , " ",self.origin[2]
		print "axis1:  ", self.axis1[0], " ",self.axis1[1] , " ",self.axis1[2]
		print "axis2:  ", self.axis2[0], " ",self.axis2[1] , " ",self.axis2[2]
		print "axis3:  ", self.axis3[0], " ",self.axis3[1] , " ",self.axis3[2]
		print ""


class md2_tags_obj:
	#Header Structure
	ident = 0		#int 0	This is used to identify the file
	version = 0		#int 1	The version number of the file (Must be 1)
	num_tags = 0		#int 2	The number of skins associated with the model
	num_frames = 0		#int 3	The number of animation frames

	offset_names = 0	#int 4	The offset in the file for the name data
	offset_tags = 0 	#int 5	The offset in the file for the tags data
	offset_end = 0		#int 6	The end of the file offset
	offset_extract_end = 0	#int 7	???
	binary_format = "<8i" 	#little-endian (<), 8 integers (8i)

	#md2 tag data objects
	names = []
	tags = [] # (consists of a number of frames)

	"""
	"tags" example:
	tags = (
		(tag1_frame1, tag1_frame2, tag1_frame3, etc...),	# tag_frames1
		(tag2_frame1, tag2_frame2, tag2_frame3, etc...),	# tag_frames2
		(tag3_frame1, tag3_frame2, tag3_frame3, etc...)		# tag_frames3
	)
	"""

	def __init__ (self):
		self.names = []
		self.tags = []
	def getSize(self):
		return struct.calcsize(self.binary_format)
	def save(self, file):
		temp_data=[0]*8
		temp_data[0] = self.ident
		temp_data[1] = self.version
		temp_data[2] = self.num_tags
		temp_data[3] = self.num_frames
		temp_data[4] = self.offset_names
		temp_data[5] = self.offset_tags
		temp_data[6] = self.offset_end
		temp_data[7] = self.offset_extract_end
		data=struct.pack(self.binary_format, temp_data[0],temp_data[1],temp_data[2],temp_data[3],temp_data[4],temp_data[5],temp_data[6],temp_data[7])
		file.write(data)

		#write the names data
		for name in self.names:
			name.save(file)
		tag_count = 0
		for tag_frames in self.tags:
			tag_count+=1
			frame_count=0
			print "Saving tag: ", tag_count

			for tag in tag_frames:
				frame_count += 1
				tag.save(file)
			print "  Frames saved: ", frame_count, "(",self.num_frames, ")"
	def load(self, file):
		temp_data = file.read(self.getSize())
		#print "TEST", temp_data# DEBUG
		data = struct.unpack(self.binary_format, temp_data)

		self.ident = data[0]
		self.version = data[1]

		if (self.ident!=844121162 or self.version!=1):
			print "Not a valid MD2 TAG file"
			Exit()

		self.num_tags		= data[2]
		self.num_frames		= data[3]
		self.offset_names	= data[4]
		#self.offset_tags	= temp_data[5]	# TODO: why was temp_data used here?
		#self.offset_end	= temp_data[6]	# TODO: why was temp_data used here?
		self.offset_tags	= data[5]
		self.offset_end		= data[6]
		self.offset_extract_end	= data[7]

		self.dump()

		# Read names data
		for tag_counter in range(0,self.num_tags):
			temp_name = md2_tagname("")
			self.names.append(temp_name.load(file))
		print "Reading of ", tag_counter, " names finished."

		for tag_counter in range(0,self.num_tags):
			frames = []
			for frame_counter in range(0,self.num_frames):
				temp_tag = md2_tag()
				temp_tag.load(file)
				#if (frame_counter < 10): #DEBUG
				frames.append(temp_tag)
			self.tags.append(frames)
			#for frame_counter in range(0,10):	#DEBUG
			#	self.tags[tag_counter][frame_counter].dump()	#DEBUG

			print " Reading of ", frame_counter+1, " frames finished."
		print "Reading of ", tag_counter+1, " framelists finished."
		return self
	def dump (self):
		print "Header Information"
		print "ident: ",		self.ident
		print "version: ",		self.version
		print "number of tags: ",	self.num_tags
		print "number of frames: ",	self.num_frames
		print "offset names: ",		self.offset_names
		print "offset tags: ",		self.offset_tags
		print "offset end: ",		self.offset_end
		print "offset extract end: ",	self.offset_extract_end
		print ""

######################################################
# Fill MD2 data structure
######################################################
def fill_md2_tags(md2_tags, object):
	global g_scale
	Blender.Window.DrawProgressBar(0.0, "Filling MD2 Data")

	# Set header information.
	md2_tags.ident=844121162
	md2_tags.version=1
	md2_tags.num_tags+=1

	# Add a name node to the tagnames data structure.
	md2_tags.names.append(md2_tagname(object.name))	# TODO: cut to 64 chars

	# Add a (empty) list of tags-positions (for each frame).
	tag_frames = []

	progress = 0.0
	progressIncrement = 1.0 / md2_tags.num_frames

	# Store currently set frame
	previous_curframe = Blender.Get("curframe")

	frame_counter = 0
	# Fill in each tag with its positions per frame
	print "Blender startframe:", Blender.Get('staframe')
	print "Blender endframe:", Blender.Get('endframe')
	for current_frame in range(Blender.Get('staframe') , Blender.Get('endframe') + 1):
		#print current_frame, "(", frame_counter, ")" # DEBUG
		progress+=progressIncrement
		Blender.Window.DrawProgressBar(progress, "Tag:"+ str(md2_tags.num_tags) +" Frame:" + str(frame_counter))

		#add a frame
		tag_frames.append(md2_tag())

		#set blender to the correct frame (so the objects have their new positions)
		Blender.Set("curframe", current_frame - 1)

		# Set first coordiantes to the location of the empty.
		tag_frames[frame_counter].origin = object.getLocation('worldspace')
		# print tag_frames[frame_counter].origin[0], " ",tag_frames[frame_counter].origin[1]," ",tag_frames[frame_counter].origin[2] # Useful for DEBUG (slowdown!)

		matrix = object.getMatrix('worldspace')
		tag_frames[frame_counter].axis1 = (matrix[0][0], matrix[0][1], matrix[0][2])
		tag_frames[frame_counter].axis2 = (matrix[1][0], matrix[1][1], matrix[1][2])
		tag_frames[frame_counter].axis3 = (matrix[2][0], matrix[2][1], matrix[2][2])
		frame_counter += 1

	# Restore curframe from before the calculation.
	Blender.Set("curframe", previous_curframe)

	md2_tags.tags.append(tag_frames)

######################################################
# Count the available tags
######################################################
def num_tags():
	mesh_objs = Blender.Object.GetSelected()
	num_tags = 0
	for object in mesh_objs:
		if object.getType()=="Empty":
			num_tags += 1

	return num_tags

######################################################
# Save MD2 TAGs Format
######################################################
def save_md2_tags(filename):
	print ""
	print "***********************************"
	print "MD2 TAG Export"
	print "***********************************"
	print ""

	Blender.Window.DrawProgressBar(0.0,"Beginning MD2 TAG Export")

	md2_tags=md2_tags_obj()	# Blank md2 object to be saved.

	#get the object
	mesh_objs = Blender.Object.GetSelected()

	#check there is a blender object selected
	if len(mesh_objs)==0:
		print "Fatal Error: Must select at least one 'Empty' to output a tag file"
		print "Found nothing"
		result=Blender.Draw.PupMenu("Must select an object to export%t|OK")
		return

	# Set frame number.
	md2_tags.num_frames = 1 + Blender.Get('endframe') - Blender.Get('staframe')

	print "Frames to export: ",md2_tags.num_frames
	print ""

	for object in mesh_objs:
		#check if it's an "Empty" mesh object
		if object.getType() != "Empty":
			print "Ignoring non-'Empty' object: ", object.getType()
		else:
			print "Found Empty: ",object.name
			fill_md2_tags(md2_tags, object)

	Blender.Window.DrawProgressBar(0.0, "Writing to Disk")

	# Set offset of names
	temp_header = md2_tags_obj();
	md2_tags.offset_names= 0+temp_header.getSize();

	# Set offset of tags
	temp_name = md2_tagname("");
	md2_tags.offset_tags = md2_tags.offset_names + temp_name.getSize() * md2_tags.num_tags;

	# Set EOF offest value.
	temp_tag = md2_tag();
	md2_tags.offset_end = md2_tags.offset_tags + (48 * md2_tags.num_frames * md2_tags.num_tags)
	md2_tags.offset_extract_end = md2_tags.offset_tags + (64 * md2_tags.num_frames * md2_tags.num_tags)

	print ""
	md2_tags.dump()

	# Actually write it to disk.
	file_export=open(filename,"wb")
	md2_tags.save(file_export)
	file_export.close()

	# Cleanup
	md2_tags=0

	Blender.Window.DrawProgressBar(1.0,"") # clear progressbar
	print "Done Writing. Closed the file"

######################################################
# Load&animate MD2 TAGs Format
######################################################
def load_md2_tags(filename):
	print ""
	print "***********************************"
	print "MD2 TAG Import"
	print "***********************************"
	print ""

	Blender.Window.DrawProgressBar(0.0,"Beginning MD2 TAG Import")

	md2_tags=md2_tags_obj()	# Blank md2 object to load to.

	print "Opening the file ..."
	Blender.Window.DrawProgressBar(0.1, "Loading from Disk")
	file_import=open(filename,"rb")
	md2_tags.load(file_import)
	file_import.close()
	print "Closed the file."

	print ""
	md2_tags.dump()

	# Get current Scene.
	scene = Scene.getCurrent()

	Blender.Window.DrawProgressBar(0.2, "Reading md2 tag data")

	progress=0.3
	progressIncrement=0.6 / (md2_tags.num_frames * md2_tags.num_tags)

	# Store currently set frame
	previous_curframe = Blender.Get("curframe")

	for tag in range(0,md2_tags.num_tags):
		# Get name & frame-data of current tag.
		tag_name = md2_tags.names[tag]
		tag_frames = md2_tags.tags[tag]

		# Create object.
		blender_tag = Object.New('Empty')
		blender_tag.setName(tag_name.name)

		# Activate name-visibility for this object.
		blender_tag.setDrawMode(blender_tag.getDrawMode() | 8)   # 8="drawname"

		# Link Object to current Scene
		scene.link(blender_tag)

		for frame_counter in range(0,md2_tags.num_frames):
		#for frame_counter in range(0,10): #DEBUG
			progress += progressIncrement
			Blender.Window.DrawProgressBar(progress, "Calculating Frame "+str(frame_counter)+" | "+tag_name.name)
			#set blender to the correct frame (so the objects' data will be set in this frame)
			Blender.Set("curframe", frame_counter+1)
			#Blender.Set("curframe", frame_counter) # use this if imported emties do not match up with imported md2 files

			# Note: Quake3 uses left-hand geometry
			origin	= tag_frames[frame_counter].origin
			forward	= tag_frames[frame_counter].axis1
			left	= tag_frames[frame_counter].axis2
			up	= tag_frames[frame_counter].axis3

			transform = MatrixSetupTransform(forward, left, up, origin)
			##print transform #DEBUG
			transform2 = Blender.Mathutils.Matrix(transform[0], transform[1], transform[2], transform[3])
			blender_tag.setMatrix(transform2)

			# Set object position.
			blender_tag.setLocation(origin[0], origin[1], origin[2])

			# Insert keyframe for current frame&object.
			blender_tag.insertIpoKey(LOCROTSIZE)

	Blender.Window.DrawProgressBar(1.0, "MD2 TAG Import")

	# Restore curframe from before the calculation.
	Blender.Set("curframe", previous_curframe)

	Blender.Window.RedrawAll()



#!BPY

"""
Name: 'MD2 (.md2) 0.16 modified to work with UFO:AI again'
Blender: 239
Group: 'Import'
Tooltip: 'Import from Quake file format (.md2).'
"""

__author__ = 'Bob Holcomb'
__version__ = '0.16-ufoai'
__url__ = ["Bob's site, http://bane.servebeer.com",
     "Support forum, http://scourage.servebeer.com/phpbb/", "blender", "blenderartists.org"]
__email__ = ["Bob Holcomb, bob_holcomb:hotmail*com", "scripts"]
__bpydoc__ = """\
This script imports a Quake 2 file (MD2), textures,
and animations into blender for editing.  Loader is based on MD2 loader from www.gametutorials.com-Thanks DigiBen! and the md3 blender loader by PhaethonH <phaethon@linux.ucla.edu><br>

 Additional help from: Shadwolf, Skandal, Rojo and Campbell Barton<br>
 Thanks Guys!
"""

# ***** BEGIN GPL LICENSE BLOCK *****
#
# Script copyright (C) Bob Holcomb
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------


import Blender
from Blender import Mesh, Object, sys
from Blender.BGL import *
from Blender.Draw import *
from Blender.Window import *
from Blender.Mathutils import Vector
import struct
from types import *


######################################################
# Main Body
######################################################

#returns the string from a null terminated string
def asciiz (s):
	n = 0
	for ch in s:
		if ord(ch) == 0:
			break;

	return s[0:n]


######################################################
# MD2 Model Constants
######################################################
MD2_MAX_TRIANGLES=4096
MD2_MAX_VERTICES=2048
MD2_MAX_TEXCOORDS=2048
MD2_MAX_FRAMES=512
MD2_MAX_SKINS=32
MD2_MAX_FRAMESIZE=(MD2_MAX_VERTICES * 4 + 128)

######################################################
# MD2 data structures
######################################################
class md2_alias_triangle(object):
	__slots__ = 'vertices', 'lightnormalindex'
	binary_format="<3BB" #little-endian (<), 3 Unsigned char

	def __init__(self):
		self.vertices=[0]*3
		self.lightnormalindex=0

	def load(self, file):
		temp_data = file.read(struct.calcsize(self.binary_format))
		data = struct.unpack(self.binary_format, temp_data)
		self.vertices[0]=data[0]
		self.vertices[1]=data[1]
		self.vertices[2]=data[2]
		self.lightnormalindex=data[3]
		return self

	def dump(self):
		print "MD2 Alias_Triangle Structure"
		print "vertex: ", self.vertices[0]
		print "vertex: ", self.vertices[1]
		print "vertex: ", self.vertices[2]
		print "lightnormalindex: ",self.lightnormalindex
		print ""

class md2_face(object):

	binary_format="<3h3h" #little-endian (<), 3 short, 3 short

	__slots__ = 'vertex_index', 'texture_index'

	def __init__(self):
		self.vertex_index = [ 0, 0, 0 ]
		self.texture_index = [ 0, 0, 0]

	def load (self, file):
		temp_data=file.read(struct.calcsize(self.binary_format))
		data=struct.unpack(self.binary_format, temp_data)
		self.vertex_index[0]=data[0]
		self.vertex_index[1]=data[1]
		self.vertex_index[2]=data[2]
		self.texture_index[0]=data[3]
		self.texture_index[1]=data[4]
		self.texture_index[2]=data[5]
		return self

	def dump (self):
		print "MD2 Face Structure"
		print "vertex index: ", self.vertex_index[0]
		print "vertex index: ", self.vertex_index[1]
		print "vertex index: ", self.vertex_index[2]
		print "texture index: ", self.texture_index[0]
		print "texture index: ", self.texture_index[1]
		print "texture index: ", self.texture_index[2]
		print ""

class md2_tex_coord(object):
	__slots__ = 'u', 'v'
	binary_format="<2h" #little-endian (<), 2 unsigned short

	def __init__(self):
		self.u=0
		self.v=0

	def load (self, file):
		temp_data=file.read(struct.calcsize(self.binary_format))
		data=struct.unpack(self.binary_format, temp_data)
		self.u=data[0]
		self.v=data[1]
		return self

	def dump (self):
		print "MD2 Texture Coordinate Structure"
		print "texture coordinate u: ",self.u
		print "texture coordinate v: ",self.v
		print ""


class md2_skin(object):
	__slots__ = 'name'
	binary_format="<64s" #little-endian (<), char[64]

	def __init__(self):
		self.name=""

	def load (self, file):
		temp_data=file.read(struct.calcsize(self.binary_format))
		data=struct.unpack(self.binary_format, temp_data)
		self.name=asciiz(data[0])
		return self

	def dump (self):
		print "MD2 Skin"
		print "skin name: ",self.name
		print ""

class md2_alias_frame(object):
	__slots__ = 'scale', 'translate', 'name', 'vertices'
	binary_format="<3f3f16s" #little-endian (<), 3 float, 3 float char[16]
	#did not add the "3bb" to the end of the binary format
	#because the alias_vertices will be read in through
	#thier own loader

	def __init__(self):
		self.scale=[0.0]*3
		self.translate=[0.0]*3
		self.name=""
		self.vertices=[]


	def load (self, file):
		temp_data=file.read(struct.calcsize(self.binary_format))
		data=struct.unpack(self.binary_format, temp_data)
		self.scale[0]=data[0]
		self.scale[1]=data[1]
		self.scale[2]=data[2]
		self.translate[0]=data[3]
		self.translate[1]=data[4]
		self.translate[2]=data[5]
		self.name=asciiz(data[6])
		return self

	def dump (self):
		print "MD2 Alias Frame"
		print "scale x: ",self.scale[0]
		print "scale y: ",self.scale[1]
		print "scale z: ",self.scale[2]
		print "translate x: ",self.translate[0]
		print "translate y: ",self.translate[1]
		print "translate z: ",self.translate[2]
		print "name: ",self.name
		print ""

class md2_obj(object):
	__slots__ =\
	'tex_coords', 'faces', 'frames',\
	'skins', 'ident', 'version',\
	'skin_width', 'skin_height',\
	'frame_size', 'num_skins', 'num_vertices',\
	'num_tex_coords', 'num_faces', 'num_GL_commands',\
	'num_frames', 'offset_skins', 'offset_tex_coords',\
	'offset_faces', 'offset_frames', 'offset_GL_commands'

	'''
	#Header Structure
	ident=0				#int 0	This is used to identify the file
	version=0			#int 1	The version number of the file (Must be 8)
	skin_width=0		#int 2	The skin width in pixels
	skin_height=0		#int 3	The skin height in pixels
	frame_size=0		#int 4	The size in bytes the frames are
	num_skins=0			#int 5	The number of skins associated with the model
	num_vertices=0		#int 6	The number of vertices (constant for each frame)
	num_tex_coords=0	#int 7	The number of texture coordinates
	num_faces=0			#int 8	The number of faces (polygons)
	num_GL_commands=0	#int 9	The number of gl commands
	num_frames=0		#int 10	The number of animation frames
	offset_skins=0		#int 11	The offset in the file for the skin data
	offset_tex_coords=0	#int 12	The offset in the file for the texture data
	offset_faces=0		#int 13	The offset in the file for the face data
	offset_frames=0		#int 14	The offset in the file for the frames data
	offset_GL_commands=0#int 15	The offset in the file for the gl commands data
	offset_end=0		#int 16	The end of the file offset
	'''
	binary_format="<17i"  #little-endian (<), 17 integers (17i)

	#md2 data objects

	def __init__ (self):
		self.tex_coords=[]
		self.faces=[]
		self.frames=[]
		self.skins=[]


	def load (self, file):
		temp_data = file.read(struct.calcsize(self.binary_format))
		data = struct.unpack(self.binary_format, temp_data)

		self.ident=data[0]
		self.version=data[1]

		if (self.ident!=844121161 or self.version!=8):
			print "Not a valid MD2 file"
			Exit()

		self.skin_width=data[2]
		self.skin_height=data[3]
		self.frame_size=data[4]

		#make the # of skin objects for model
		self.num_skins=data[5]
		for i in xrange(0,self.num_skins):
			self.skins.append(md2_skin())

		self.num_vertices=data[6]

		#make the # of texture coordinates for model
		self.num_tex_coords=data[7]
		for i in xrange(0,self.num_tex_coords):
			self.tex_coords.append(md2_tex_coord())

		#make the # of triangle faces for model
		self.num_faces=data[8]
		for i in xrange(0,self.num_faces):
			self.faces.append(md2_face())

		self.num_GL_commands=data[9]

		#make the # of frames for the model
		self.num_frames=data[10]
		for i in xrange(0,self.num_frames):
			self.frames.append(md2_alias_frame())
			#make the # of vertices for each frame
			for j in xrange(0,self.num_vertices):
				self.frames[i].vertices.append(md2_alias_triangle())

		self.offset_skins=data[11]
		self.offset_tex_coords=data[12]
		self.offset_faces=data[13]
		self.offset_frames=data[14]
		self.offset_GL_commands=data[15]

		#load the skin info
		file.seek(self.offset_skins,0)
		for i in xrange(0, self.num_skins):
			self.skins[i].load(file)
			#self.skins[i].dump()

		#load the texture coordinates
		file.seek(self.offset_tex_coords,0)
		for i in xrange(0, self.num_tex_coords):
			self.tex_coords[i].load(file)
			#self.tex_coords[i].dump()

		#load the face info
		file.seek(self.offset_faces,0)
		for i in xrange(0, self.num_faces):
			self.faces[i].load(file)
			#self.faces[i].dump()

		#load the frames
		file.seek(self.offset_frames,0)
		for i in xrange(0, self.num_frames):
			self.frames[i].load(file)
			#self.frames[i].dump()
			for j in xrange(0,self.num_vertices):
				self.frames[i].vertices[j].load(file)
				#self.frames[i].vertices[j].dump()
		return self

	def dump (self):
		print "Header Information"
		print "ident: ", self.ident
		print "version: ", self.version
		print "skin width: ", self.skin_width
		print "skin height: ", self.skin_height
		print "frame size: ", self.frame_size
		print "number of skins: ", self.num_skins
		print "number of texture coordinates: ", self.num_tex_coords
		print "number of faces: ", self.num_faces
		print "number of frames: ", self.num_frames
		print "number of vertices: ", self.num_vertices
		print "offset skins: ", self.offset_skins
		print "offset texture coordinates: ", self.offset_tex_coords
		print "offset faces: ", self.offset_faces
		print "offset frames: ",self.offset_frames
		print ""

######################################################
# Import functions
######################################################
def load_textures(md2, texture_filename):
	#did the user specify a texture they wanted to use?
	if texture_filename:
		if (Blender.sys.exists(texture_filename)):
			try:	return Blender.Image.Load(texture_filename)
			except:	return -1 # could not load?

	#does the model have textures specified with it?
	if int(md2.num_skins) > 0:
		for i in xrange(0,md2.num_skins):
			#md2.skins[i].dump()
			if (Blender.sys.exists(md2.skins[i].name)):
				try:	return Blender.Image.Load(md2.skins[i].name)
				except:	return -1


def animate_md2(md2, mesh):
	######### Animate the verts through keyframe animation

	# Fast access to the meshes vertex coords
	verts = [v.co for v in mesh.verts]
	scale = g_scale.val

	for i in xrange(1, md2.num_frames):
		frame = md2.frames[i]
		#update the vertices
		for j in xrange(md2.num_vertices):
			x=(frame.scale[0] * frame.vertices[j].vertices[0] + frame.translate[0]) * scale
			y=(frame.scale[1] * frame.vertices[j].vertices[1] + frame.translate[1]) * scale
			z=(frame.scale[2] * frame.vertices[j].vertices[2] + frame.translate[2]) * scale

			#put the vertex in the right spot
			verts[j][:] = y,-x,z

		mesh.insertKey(i,"absolute")
		# mesh.insertKey(i)

		#not really necissary, but I like playing with the frame counter
		Blender.Set("curframe", i)


	# Make the keys animate in the 3d view.
	key = mesh.key
	key.relative = False

	# Add an IPO to teh Key
	ipo = Blender.Ipo.New('Key', 'md2')
	key.ipo = ipo
	# Add a curve to the IPO
	curve = ipo.addCurve('Basis')

	# Add 2 points to cycle through the frames.
	curve.append((1, 0))
	curve.append((md2.num_frames, (md2.num_frames-1)/10.0))
	curve.interpolation = Blender.IpoCurve.InterpTypes.LINEAR



def load_md2(md2_filename, texture_filename):
	#read the file in
	file=open(md2_filename,"rb")
	WaitCursor(1)
	DrawProgressBar(0.0, 'Loading MD2')
	md2=md2_obj()
	md2.load(file)
	#md2.dump()
	file.close()

	######### Creates a new mesh
	mesh = Mesh.New()

	uv_coord=[]
	#uv_list=[]
	verts_extend = []
	#load the textures to use later
	#-1 if there is no texture to load
	mesh_image=load_textures(md2, texture_filename)
	if mesh_image == -1 and texture_filename:
		print 'MD2 Import, Warning, texture "%s" could not load'

	######### Make the verts
	DrawProgressBar(0.25,"Loading Vertex Data")
	frame = md2.frames[0]
	scale = g_scale.val

	def tmp_get_vertex(i):
		#use the first frame for the mesh vertices
		x=(frame.scale[0]*frame.vertices[i].vertices[0]+frame.translate[0])*scale
		y=(frame.scale[1]*frame.vertices[i].vertices[1]+frame.translate[1])*scale
		z=(frame.scale[2]*frame.vertices[i].vertices[2]+frame.translate[2])*scale
		return y,-x,z

	mesh.verts.extend( [tmp_get_vertex(i) for i in xrange(0,md2.num_vertices)] )
	del tmp_get_vertex

	######## Make the UV list
	DrawProgressBar(0.50,"Loading UV Data")

	w = float(md2.skin_width)
	h = float(md2.skin_height)
	if w <= 0.0: w = 1.0
	if h <= 0.0: h = 1.0
	#for some reason quake2 texture maps are upside down, flip that
	uv_list = [Vector(co.u/w, 1-(co.v/h)) for co in md2.tex_coords]
	del w, h

	######### Make the faces
	DrawProgressBar(0.75,"Loading Face Data")
	faces = []
	face_uvs = []
	for md2_face in md2.faces:
		f = md2_face.vertex_index[0], md2_face.vertex_index[2], md2_face.vertex_index[1]
		uv = uv_list[md2_face.texture_index[0]], uv_list[md2_face.texture_index[2]], uv_list[md2_face.texture_index[1]]

		if f[2] == 0:
			# EEKADOODLE :/
			f= f[1], f[2], f[0]
			uv= uv[1], uv[2], uv[0]

		#ditto in reverse order with the texture verts
		faces.append(f)
		face_uvs.append(uv)


	face_mapping = mesh.faces.extend(faces, indexList=True)
	print len(faces)
	print len(mesh.faces)
	mesh.faceUV= True  #turn on face UV coordinates for this mesh
	mesh_faces = mesh.faces
	for i, uv in enumerate(face_uvs):
		if face_mapping[i] != None:
			f = mesh_faces[face_mapping[i]]
			f.uv = uv
			if (mesh_image!=-1):
				f.image=mesh_image

	scn= Blender.Scene.GetCurrent()
	mesh_obj= scn.objects.new(mesh)
	animate_md2(md2, mesh)
	DrawProgressBar(0.98,"Loading Animation Data")

	#locate the Object containing the mesh at the cursor location
	cursor_pos=Blender.Window.GetCursorPos()
	mesh_obj.setLocation(float(cursor_pos[0]),float(cursor_pos[1]),float(cursor_pos[2]))
	DrawProgressBar (1.0, "")
	WaitCursor(0)

#***********************************************
# MAIN
#***********************************************

# Import globals
g_md2_filename=Create("*.md2")
#g_md2_filename=Create("/d/warvet/tris.md2")
g_texture_filename=Create('')
# g_texture_filename=Create("/d/warvet/warvet.jpg")

g_filename_search=Create("*.md2")
g_texture_search=Create('')
# g_texture_search=Create("/d/warvet/warvet.jpg")

#Globals
g_scale=Create(1.0)

# Events
EVENT_NOEVENT=1
EVENT_LOAD_MD2=2
EVENT_CHOOSE_FILENAME=3
EVENT_CHOOSE_TEXTURE=4
EVENT_SAVE_MD2=5
EVENT_EXIT=100

######################################################
# Callbacks for Window functions
######################################################
def filename_callback(input_filename):
	global g_md2_filename
	g_md2_filename.val=input_filename

def texture_callback(input_texture):
	global g_texture_filename
	g_texture_filename.val=input_texture

######################################################
# GUI Loader
######################################################


def draw_gui():
	global g_scale
	global g_md2_filename
	global g_texture_filename
	global EVENT_NOEVENT,EVENT_LOAD_MD2,EVENT_CHOOSE_FILENAME,EVENT_CHOOSE_TEXTURE,EVENT_EXIT

	########## Titles
	glClear(GL_COLOR_BUFFER_BIT)
	glRasterPos2d(8, 125)
	Text("MD2 loader")

	######### Parameters GUI Buttons
	BeginAlign()
	g_md2_filename = String("MD2 file to load: ", EVENT_NOEVENT, 10, 55, 210, 18,
							g_md2_filename.val, 255, "MD2 file to load")
	########## MD2 File Search Button
	Button("Browse",EVENT_CHOOSE_FILENAME,220,55,80,18)
	EndAlign()

	BeginAlign()
	g_texture_filename = String("Texture file to load: ", EVENT_NOEVENT, 10, 35, 210, 18,
								g_texture_filename.val, 255, "Texture file to load-overrides MD2 file")
	########## Texture Search Button
	Button("Browse",EVENT_CHOOSE_TEXTURE,220,35,80,18)
	EndAlign()

	########## Scale slider-default is 1/8 which is a good scale for md2->blender
	g_scale= Slider("Scale Factor: ", EVENT_NOEVENT, 10, 75, 210, 18,
					1.0, 0.001, 10.0, 1, "Scale factor for obj Model");

	######### Draw and Exit Buttons
	Button("Load",EVENT_LOAD_MD2 , 10, 10, 80, 18)
	Button("Exit",EVENT_EXIT , 170, 10, 80, 18)

def event(evt, val):
	if (evt == QKEY and not val):
		Blender.Draw.Exit()

def bevent(evt):
	global g_md2_filename
	global g_texture_filename
	global EVENT_NOEVENT,EVENT_LOAD_MD2,EVENT_SAVE_MD2,EVENT_EXIT

	######### Manages GUI events
	if (evt==EVENT_EXIT):
		Blender.Draw.Exit()
	elif (evt==EVENT_CHOOSE_FILENAME):
		FileSelector(filename_callback, "MD2 File Selection")
	elif (evt==EVENT_CHOOSE_TEXTURE):
		FileSelector(texture_callback, "Texture Selection")
	elif (evt==EVENT_LOAD_MD2):
		if not Blender.sys.exists(g_md2_filename.val):
			PupMenu('Model file does not exist')
			return
		else:
			load_md2(g_md2_filename.val, g_texture_filename.val)
			Blender.Redraw()
			Blender.Draw.Exit()
			return

if __name__ == '__main__':
	Register(draw_gui, event, bevent)
